@@ -1,5 +1,5 @@
 /*
- * $Id: autocompletion.c 6463 2008-07-27 21:38:24Z dothebart $
+ * $Id: autocompletion.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * ajax-powered autocompletion...
  */
@@ -25,22 +25,22 @@ void recp_autocomplete(char *partial) {
 		PACKAGE_STRING);
 	begin_burst();
 
-	wprintf("<ul>");
+	wc_printf("<ul>");
 
 	serv_printf("AUTO %s", partial);
 	serv_getln(buf, sizeof buf);
 	if (buf[0] == '1') {
 		while(serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
 			extract_token(name, buf, 0, '|', sizeof name);
-			wprintf("<li>");
+			wc_printf("<li>");
 			escputs(name);
-			wprintf("</li>");
+			wc_printf("</li>");
 		}
 	}
 
-	wprintf("</ul>");
+	wc_printf("</ul>");
 
-	wprintf("\r\n\r\n");
+	wc_printf("\r\n\r\n");
 	wDumpContent(0);
 }
 
@@ -54,7 +54,7 @@ void
 InitModule_AUTO_COMPLETE
 (void)
 {
-	WebcitAddUrlHandler(HKEY("recp_autocomplete"), _recp_autocomplete, 0);
-	WebcitAddUrlHandler(HKEY("cc_autocomplete"),   _cc_autocomplete, 0);
-	WebcitAddUrlHandler(HKEY("bcc_autocomplete"),  _bcc_autocomplete, 0);
+	WebcitAddUrlHandler(HKEY("recp_autocomplete"), "", 0, _recp_autocomplete, 0);
+	WebcitAddUrlHandler(HKEY("cc_autocomplete"),   "", 0, _cc_autocomplete, 0);
+	WebcitAddUrlHandler(HKEY("bcc_autocomplete"),  "", 0, _bcc_autocomplete, 0);
 }
