@@ -1,34 +1,26 @@
 /*
- * $Id: crypto.c 6648 2008-10-01 19:48:50Z dothebart $
- */
-/**
- * \defgroup https  Provides HTTPS, when the OpenSSL library is available.
- * \ingroup WebcitHttpServer 
+ * $Id: crypto.c 7819 2009-09-23 21:22:30Z ajc $
  */
 
-/*@{*/
 #include "sysdep.h"
 #ifdef HAVE_OPENSSL
 
 #include "webcit.h"
 #include "webserver.h"
-/** \todo dirify */
-/** where to find the keys */
+
+/* where to find the keys */
 #define	CTDL_CRYPTO_DIR		ctdl_key_dir
-#define CTDL_KEY_PATH		file_crpt_file_key /**< the key */
-#define CTDL_CSR_PATH		file_crpt_file_csr /**< the csr file */
-#define CTDL_CER_PATH		file_crpt_file_cer /**< the cer file */
-#define SIGN_DAYS		365 /**< how long our certificate should live */
+#define CTDL_KEY_PATH		file_crpt_file_key
+#define CTDL_CSR_PATH		file_crpt_file_csr
+#define CTDL_CER_PATH		file_crpt_file_cer
+#define SIGN_DAYS		3650			/* how long our certificate should live */
 
-SSL_CTX *ssl_ctx;		/**< SSL context */
-pthread_mutex_t **SSLCritters;	/**< Things needing locking */
+SSL_CTX *ssl_ctx;		/* SSL context */
+pthread_mutex_t **SSLCritters;	/* Things needing locking */
+char *ssl_cipher_list = DEFAULT_SSL_CIPHER_LIST;
 
-pthread_key_t ThreadSSL;	/**< Per-thread SSL context */
+pthread_key_t ThreadSSL;	/* Per-thread SSL context */
 
-/**
- * \brief what?????
- * \return thread id??? 
- */
 static unsigned long id_callback(void)
 {
 	return (unsigned long) pthread_self();
@@ -44,12 +36,11 @@ void shutdown_ssl(void)
 	 * for (i = 0; i < n; i++)
 	 * 	free(SSLCritters[i]);
 	 * free(SSLCritters);
-	*/
+	 */
 }
 
-/**
- * \brief initialize ssl engine
- * load certs and initialize openssl internals
+/*
+ * initialize ssl engine, load certs and initialize openssl internals
  */
 void init_ssl(void)
 {
@@ -62,17 +53,17 @@ void init_ssl(void)
 	X509_NAME *name = NULL;
 	FILE *fp;
 	char buf[SIZ];
+	int rv = 0;
 
-	if (!access("/var/run/egd-pool", F_OK))
+	if (!access("/var/run/egd-pool", F_OK)) {
 		RAND_egd("/var/run/egd-pool");
+	}
 
 	if (!RAND_status()) {
-		lprintf(3,
-			"PRNG not adequately seeded, won't do SSL/TLS\n");
+		lprintf(3, "PRNG not adequately seeded, won't do SSL/TLS\n");
 		return;
 	}
-	SSLCritters =
-	    malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t *));
+	SSLCritters = malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t *));
 	if (!SSLCritters) {
 		lprintf(1, "citserver: can't allocate memory!!\n");
 		/* Nothing's been initialized, just die */
@@ -94,71 +85,79 @@ void init_ssl(void)
 		}
 	}
 
-	/**
+	/*
 	 * Initialize SSL transport layer
 	 */
 	SSL_library_init();
 	SSL_load_error_strings();
 	ssl_method = SSLv23_server_method();
 	if (!(ssl_ctx = SSL_CTX_new(ssl_method))) {
-		lprintf(3, "SSL_CTX_new failed: %s\n",
-			ERR_reason_error_string(ERR_get_error()));
+		lprintf(3, "SSL_CTX_new failed: %s\n", ERR_reason_error_string(ERR_get_error()));
+		return;
+	}
+
+	lprintf(9, "Requesting cipher list: %s\n", ssl_cipher_list);
+	if (!(SSL_CTX_set_cipher_list(ssl_ctx, ssl_cipher_list))) {
+		lprintf(3, "SSL_CTX_set_cipher_list failed: %s\n", ERR_reason_error_string(ERR_get_error()));
 		return;
 	}
 
 	CRYPTO_set_locking_callback(ssl_lock);
 	CRYPTO_set_id_callback(id_callback);
 
-	/**
-	 * Get our certificates in order. \todo dirify. this is a setup job.
+	/*
+	 * Get our certificates in order. (FIXME: dirify. this is a setup job.)
 	 * First, create the key/cert directory if it's not there already...
 	 */
 	mkdir(CTDL_CRYPTO_DIR, 0700);
 
-	/**
+	/*
 	 * Before attempting to generate keys/certificates, first try
 	 * link to them from the Citadel server if it's on the same host.
 	 * We ignore any error return because it either meant that there
 	 * was nothing in Citadel to link from (in which case we just
 	 * generate new files) or the target files already exist (which
-	 * is not fatal either). \todo dirify
+	 * is not fatal either).
 	 */
 	if (!strcasecmp(ctdlhost, "uds")) {
 		sprintf(buf, "%s/keys/citadel.key", ctdlport);
-		symlink(buf, CTDL_KEY_PATH);
+		rv = symlink(buf, CTDL_KEY_PATH);
+		if (!rv) lprintf(1, "%s\n", strerror(errno));
 		sprintf(buf, "%s/keys/citadel.csr", ctdlport);
-		symlink(buf, CTDL_CSR_PATH);
+		rv = symlink(buf, CTDL_CSR_PATH);
+		if (!rv) lprintf(1, "%s\n", strerror(errno));
 		sprintf(buf, "%s/keys/citadel.cer", ctdlport);
-		symlink(buf, CTDL_CER_PATH);
+		rv = symlink(buf, CTDL_CER_PATH);
+		if (!rv) lprintf(1, "%s\n", strerror(errno));
 	}
 
-	/**
+	/*
 	 * If we still don't have a private key, generate one.
 	 */
 	if (access(CTDL_KEY_PATH, R_OK) != 0) {
 		lprintf(5, "Generating RSA key pair.\n");
-		rsa = RSA_generate_key(1024,	/**< modulus size */
-					65537,	/**< exponent */
-					NULL,	/**< no callback */
-					NULL);	/**< no callback */
+		rsa = RSA_generate_key(1024,	/* modulus size */
+					65537,	/* exponent */
+					NULL,	/* no callback */
+					NULL	/* no callback */
+		);
 		if (rsa == NULL) {
-			lprintf(3, "Key generation failed: %s\n",
-				ERR_reason_error_string(ERR_get_error()));
+			lprintf(3, "Key generation failed: %s\n", ERR_reason_error_string(ERR_get_error()));
 		}
 		if (rsa != NULL) {
 			fp = fopen(CTDL_KEY_PATH, "w");
 			if (fp != NULL) {
 				chmod(CTDL_KEY_PATH, 0600);
-				if (PEM_write_RSAPrivateKey(fp,	/**< the file */
-							rsa,	/**< the key */
-							NULL,	/**< no enc */
-							NULL,	/**< no passphr */
-							0,	/**< no passphr */
-							NULL,	/**< no callbk */
-							NULL	/**< no callbk */
+				if (PEM_write_RSAPrivateKey(fp,	/* the file */
+							rsa,	/* the key */
+							NULL,	/* no enc */
+							NULL,	/* no passphr */
+							0,	/* no passphr */
+							NULL,	/* no callbk */
+							NULL	/* no callbk */
 				) != 1) {
 					lprintf(3, "Cannot write key: %s\n",
-                                		ERR_reason_error_string(ERR_get_error()));
+						ERR_reason_error_string(ERR_get_error()));
 					unlink(CTDL_KEY_PATH);
 				}
 				fclose(fp);
@@ -180,7 +179,7 @@ void init_ssl(void)
 	if ( (access(CTDL_CER_PATH, R_OK) != 0) && (access(CTDL_CSR_PATH, R_OK) != 0) ) {
 		lprintf(5, "Generating a certificate signing request.\n");
 
-		/**
+		/*
 		 * Read our key from the file.  No, we don't just keep this
 		 * in memory from the above key-generation function, because
 		 * there is the possibility that the key was already on disk
@@ -198,51 +197,71 @@ void init_ssl(void)
 			if (pk=EVP_PKEY_new(), pk != NULL) {
 				EVP_PKEY_assign_RSA(pk, rsa);
 				if (req = X509_REQ_new(), req != NULL) {
-
-					/** Set the public key */
+					const char *env;
+					/* Set the public key */
 					X509_REQ_set_pubkey(req, pk);
 					X509_REQ_set_version(req, 0L);
 
 					name = X509_REQ_get_subject_name(req);
 
-					/** Tell it who we are */
+					/* Tell it who we are */
 
-					/* \todo whats this?
+					/*
+					 * We used to add these fields to the subject, but
+					 * now we don't.  Someone doing this for real isn't
+					 * going to use the webcit-generated CSR anyway.
+					 *
 					X509_NAME_add_entry_by_txt(name, "C",
 						MBSTRING_ASC, "US", -1, -1, 0);
-
+					*
 					X509_NAME_add_entry_by_txt(name, "ST",
 						MBSTRING_ASC, "New York", -1, -1, 0);
-
+					*
 					X509_NAME_add_entry_by_txt(name, "L",
 						MBSTRING_ASC, "Mount Kisco", -1, -1, 0);
 					*/
 
+					env = getenv("O");
+					if (env == NULL)
+						env = "Organization name",
+
 					X509_NAME_add_entry_by_txt(
 						name, "O",
 						MBSTRING_ASC, 
-						(unsigned char*)"Organization name",
-						-1, -1, 0);
+						(unsigned char*)env, 
+						-1, -1, 0
+					);
+
+					env = getenv("OU");
+					if (env == NULL)
+						env = "Citadel server";
 
 					X509_NAME_add_entry_by_txt(
 						name, "OU",
 						MBSTRING_ASC, 
-						(unsigned char*)"Citadel server1",
-						-1, -1, 0);
+						(unsigned char*)env, 
+						-1, -1, 0
+					);
+
+					env = getenv("CN");
+					if (env == NULL)
+						env = "*";
 
 					X509_NAME_add_entry_by_txt(
 						name, "CN",
 						MBSTRING_ASC, 
-						(unsigned char*)"*", -1, -1, 0);
+						(unsigned char*)env,
+						-1, -1, 0
+					);
 				
 					X509_REQ_set_subject_name(req, name);
 
-					/** Sign the CSR */
+					/* Sign the CSR */
 					if (!X509_REQ_sign(req, pk, EVP_md5())) {
 						lprintf(3, "X509_REQ_sign(): error\n");
 					}
 					else {
-						/** Write it to disk. */	
+						/* Write it to disk. */	
 						fp = fopen(CTDL_CSR_PATH, "w");
 						if (fp != NULL) {
 							chmod(CTDL_CSR_PATH, 0600);
@@ -270,13 +289,13 @@ void init_ssl(void)
 
 
 
-	/**
+	/*
 	 * Generate a self-signed certificate if we don't have one.
 	 */
 	if (access(CTDL_CER_PATH, R_OK) != 0) {
 		lprintf(5, "Generating a self-signed certificate.\n");
 
-		/** Same deal as before: always read the key from disk because
+		/* Same deal as before: always read the key from disk because
 		 * it may or may not have just been generated.
 		 */
 		fp = fopen(CTDL_KEY_PATH, "r");
@@ -285,7 +304,7 @@ void init_ssl(void)
 			fclose(fp);
 		}
 
-		/** This also holds true for the CSR. */
+		/* This also holds true for the CSR. */
 		req = NULL;
 		cer = NULL;
 		pk = NULL;
@@ -313,12 +332,12 @@ void init_ssl(void)
 					X509_set_pubkey(cer, req_pkey);
 					EVP_PKEY_free(req_pkey);
 					
-					/** Sign the cert */
+					/* Sign the cert */
 					if (!X509_sign(cer, pk, EVP_md5())) {
 						lprintf(3, "X509_sign(): error\n");
 					}
 					else {
-						/** Write it to disk. */	
+						/* Write it to disk. */	
 						fp = fopen(CTDL_CER_PATH, "w");
 						if (fp != NULL) {
 							chmod(CTDL_CER_PATH, 0600);
@@ -339,7 +358,7 @@ void init_ssl(void)
 		}
 	}
 
-	/**
+	/*
 	 * Now try to bind to the key and certificate.
 	 * Note that we use SSL_CTX_use_certificate_chain_file() which allows
 	 * the certificate file to contain intermediate certificates.
@@ -354,10 +373,8 @@ void init_ssl(void)
 }
 
 
-/**
- * \brief starts SSL/TLS encryption for the current session.
- * \param sock the socket connection
- * \return Zero if the SSL/TLS handshake succeeded, non-zero otherwise.
+/*
+ * starts SSL/TLS encryption for the current session.
  */
 int starttls(int sock) {
 	int retval, bits, alg_bits, r;
@@ -369,19 +386,17 @@ int starttls(int sock) {
 		return(1);
 	}
 	if (!(newssl = SSL_new(ssl_ctx))) {
-		lprintf(3, "SSL_new failed: %s\n",
-				ERR_reason_error_string(ERR_get_error()));
+		lprintf(3, "SSL_new failed: %s\n", ERR_reason_error_string(ERR_get_error()));
 		return(2);
 	}
 	if (!(SSL_set_fd(newssl, sock))) {
-		lprintf(3, "SSL_set_fd failed: %s\n",
-			ERR_reason_error_string(ERR_get_error()));
+		lprintf(3, "SSL_set_fd failed: %s\n", ERR_reason_error_string(ERR_get_error()));
 		SSL_free(newssl);
 		return(3);
 	}
 	retval = SSL_accept(newssl);
 	if (retval < 1) {
-		/**
+		/*
 		 * Can't notify the client of an error here; they will
 		 * discover the problem at the SSL layer and should
 		 * revert to unencrypted communications.
@@ -391,10 +406,12 @@ int starttls(int sock) {
 
 		errval = SSL_get_error(newssl, retval);
 		ssl_error_reason = ERR_reason_error_string(ERR_get_error());
-		if (ssl_error_reason == NULL)
-			lprintf(3, "SSL_accept failed: errval=%i, retval=%i\n", errval, retval);
-		else
+		if (ssl_error_reason == NULL) {
+			lprintf(3, "SSL_accept failed: errval=%i, retval=%i %s\n", errval, retval, strerror(errval));
+		}
+		else {
 			lprintf(3, "SSL_accept failed: %s\n", ssl_error_reason);
+		}
 		sleeeeeeeeeep(1);
 		retval = SSL_accept(newssl);
 	}
@@ -404,14 +421,19 @@ int starttls(int sock) {
 
 		errval = SSL_get_error(newssl, retval);
 		ssl_error_reason = ERR_reason_error_string(ERR_get_error());
-		if (ssl_error_reason == NULL)
-			lprintf(3, "SSL_accept failed: errval=%i, retval=%i\n", errval, retval);
-		else
+		if (ssl_error_reason == NULL) {
+			lprintf(3, "SSL_accept failed: errval=%i, retval=%i (%s)\n", errval, retval, strerror(errval));
+		}
+		else {
 			lprintf(3, "SSL_accept failed: %s\n", ssl_error_reason);
+		}
 		SSL_free(newssl);
 		newssl = NULL;
 		return(4);
-	} else lprintf(15, "SSL_accept success\n");
+	}
+	else {
+		lprintf(15, "SSL_accept success\n");
+	}
 	r = BIO_set_close(newssl->rbio, BIO_NOCLOSE);
 	bits = SSL_CIPHER_get_bits(SSL_get_current_cipher(newssl), &alg_bits);
 	lprintf(15, "SSL/TLS using %s on %s (%d of %d bits)\n",
@@ -426,8 +448,8 @@ int starttls(int sock) {
 
 
 
-/**
- * \brief shuts down the TLS connection
+/*
+ * shuts down the TLS connection
  *
  * WARNING:  This may make your session vulnerable to a known plaintext
  * attack in the current implmentation.
@@ -442,7 +464,7 @@ void endtls(void)
 	SSL_shutdown(THREADSSL);
 	ctx = SSL_get_SSL_CTX(THREADSSL);
 
-	/** I don't think this is needed, and it crashes the server anyway
+	/* I don't think this is needed, and it crashes the server anyway
 	 *
 	 * 	if (ctx != NULL) {
 	 *		lprintf(9, "Freeing CTX at %x\n", (int)ctx );
@@ -455,25 +477,21 @@ void endtls(void)
 }
 
 
-/**
- * \brief callback for OpenSSL mutex locks
- * \param mode which mode??????
- * \param n  how many???
- * \param file which filename ???
- * \param line what line????
+/*
+ * callback for OpenSSL mutex locks
  */
 void ssl_lock(int mode, int n, const char *file, int line)
 {
-	if (mode & CRYPTO_LOCK)
+	if (mode & CRYPTO_LOCK) {
 		pthread_mutex_lock(SSLCritters[n]);
-	else
+	}
+	else {
 		pthread_mutex_unlock(SSLCritters[n]);
+	}
 }
 
-/**
- * \brief Send binary data to the client encrypted.
- * \param buf chars to send to the client
- * \param nbytes how many chars
+/*
+ * Send binary data to the client encrypted.
  */
 void client_write_ssl(const StrBuf *Buf)
 {
@@ -500,8 +518,7 @@ void client_write_ssl(const StrBuf *Buf)
 			long errval;
 
 			errval = SSL_get_error(THREADSSL, retval);
-			if (errval == SSL_ERROR_WANT_READ ||
-			    errval == SSL_ERROR_WANT_WRITE) {
+			if (errval == SSL_ERROR_WANT_READ || errval == SSL_ERROR_WANT_WRITE) {
 				sleeeeeeeeeep(1);
 				continue;
 			}
@@ -517,22 +534,12 @@ void client_write_ssl(const StrBuf *Buf)
 }
 
 
-/**
- * \brief read data from the encrypted layer.
- * \param buf charbuffer to read to 
- * \param bytes how many
- * \param timeout how long should we wait?
- * \returns what???
+/*
+ * read data from the encrypted layer.
  */
 int client_read_sslbuffer(StrBuf *buf, int timeout)
 {
-#if 0
-	fd_set rfds;
-	struct timeval tv;
-	int retval;
-	int s;
-#endif
-	char sbuf[16384]; /**< Openssl communicates in 16k blocks, so lets speak its native tongue. */
+	char sbuf[16384]; /* OpenSSL communicates in 16k blocks, so let's speak its native tongue. */
 	int rlen;
 	char junk[1];
 	SSL *pssl = THREADSSL;
@@ -540,24 +547,6 @@ int client_read_sslbuffer(StrBuf *buf, int timeout)
 	if (pssl == NULL) return(-1);
 
 	while (1) {
-#if 0
-		/**
-		 * This code is disabled because we don't need it when
-		 * using blocking reads (which we are). -IO
-		 */
-		FD_ZERO(&rfds);
-		s = BIO_get_fd(pssl->rbio, NULL);
-		FD_SET(s, &rfds);
-		tv.tv_sec = timeout;
-		tv.tv_usec = 0;
-
-		retval = select(s + 1, &rfds, NULL, NULL, &tv);
-
-		if (FD_ISSET(s, &rfds) == 0) {
-			return (0);
-		}
-
-#endif
 		if (SSL_want_read(pssl)) {
 			if ((SSL_write(pssl, junk, 0)) < 1) {
 				lprintf(9, "SSL_write in client_read\n");
@@ -568,8 +557,7 @@ int client_read_sslbuffer(StrBuf *buf, int timeout)
 			long errval;
 
 			errval = SSL_get_error(pssl, rlen);
-			if (errval == SSL_ERROR_WANT_READ ||
-			    errval == SSL_ERROR_WANT_WRITE) {
+			if (errval == SSL_ERROR_WANT_READ || errval == SSL_ERROR_WANT_WRITE) {
 				sleeeeeeeeeep(1);
 				continue;
 			}
@@ -583,6 +571,4 @@ int client_read_sslbuffer(StrBuf *buf, int timeout)
 	return (0);
 }
 
-
 #endif				/* HAVE_OPENSSL */
-/*@}*/
