@@ -1,13 +1,14 @@
 /*
- * $Id: gettext.c 7169 2009-02-26 22:55:28Z dothebart $
+ * $Id: gettext.c 8168 2009-12-29 18:07:08Z dothebart $
  */
 
 #include "webcit.h"
 #include "webserver.h"
+#define SEARCH_LANG 20		/* how many langs should we parse? */
 
 #ifdef ENABLE_NLS
 /* actual supported locales */
-const char *AvailLang[NUM_LANGS] = {
+const char *AvailLang[] = {
 	"C",
 	"en_US",
 	"de_DE",
@@ -17,14 +18,18 @@ const char *AvailLang[NUM_LANGS] = {
 	"da_DK",
 	"fr_FR",
 	"nl_NL",
-	"pt_BR"
+	"pt_BR",
+	"hu_HU",
+	"et_EE",
+	"ru_RU",
+	""
 };
 
-const char *AvailLangLoaded[NUM_LANGS];
+const char **AvailLangLoaded;
 long nLocalesLoaded = 0;
 
 #ifdef HAVE_USELOCALE
-locale_t wc_locales[NUM_LANGS]; /**< here we keep the parsed stuff */
+locale_t *wc_locales; /**< here we keep the parsed stuff */
 #endif
 
 /** Keep information about one locale */
@@ -47,7 +52,7 @@ typedef struct _lang_pref{
  * \param LocaleString the string from the browser http headers
  */
 
-void httplang_to_locale(StrBuf *LocaleString)
+void httplang_to_locale(StrBuf *LocaleString, wcsession *sess)
 {
 	LangStruct wanted_locales[SEARCH_LANG];
 	LangStruct *ls;
@@ -61,11 +66,11 @@ void httplang_to_locale(StrBuf *LocaleString)
 	int nParts;
 	StrBuf *Buf = NULL;
 	StrBuf *SBuf = NULL;
-	
-	nParts=StrBufNum_tokens(LocaleString,',');
-	for (i=0; ((i<nParts)&&(i<SEARCH_LANG)); i++)
+
+	nParts = StrBufNum_tokens(LocaleString, ',');
+	for (i=0; ((i<nParts) && (i < SEARCH_LANG)); i++)
         {
-		char lbuf[16];
+		char lbuf[32];
 		int blen;
 			
 		if (Buf == NULL) {
@@ -77,82 +82,98 @@ void httplang_to_locale(StrBuf *LocaleString)
 			FlushStrBuf(SBuf);
 		}
 
-		ls=&wanted_locales[i];
+		ls = &wanted_locales[i];
 
-		StrBufExtract_token(Buf,LocaleString, i,',');
+		StrBufExtract_token(Buf, LocaleString, i, ',');
 		/** we are searching, if this list item has something like ;q=n*/
-		if (StrBufNum_tokens(Buf,'=')>1) {
+		if (StrBufNum_tokens(Buf, '=') > 1) {
 			int sbuflen, k;
-			StrBufExtract_token(SBuf,Buf, 1,'=');
-			sbuflen=StrLength(SBuf);
-			for (k=0; k<sbuflen; k++) 
-				if (ChrPtr(SBuf)[k]=='.') 
+			StrBufExtract_token(SBuf, Buf, 1, '=');
+			sbuflen = StrLength(SBuf);
+			for (k = 0; k < sbuflen; k++) 
+				if (ChrPtr(SBuf)[k] == '.') 
 					StrBufPeek(SBuf, NULL, k, '0');
-			ls->priority=StrTol(SBuf);
+			ls->priority = StrTol(SBuf);
 		}
 		else {
-			ls->priority=1000;
+			ls->priority = 1000;
 		}
+
 		/** get the locale part */
-		StrBufExtract_token(SBuf ,Buf, 0, ';');
+		StrBufExtract_token(SBuf, Buf, 0, ';');
+
 		/** get the lang part, which should be allways there */
-		extract_token(&ls->lang[0], ChrPtr(SBuf), 0, '-', 16);
+		extract_token(&ls->lang[0], 
+			      ChrPtr(SBuf), 
+			      0, '-', 
+			      sizeof(ls->lang));
+
 		/** get the area code if any. */
-		if (StrBufNum_tokens(SBuf,'-') > 1) {
-			extract_token(&ls->region[0],ChrPtr(SBuf),1,'-',16);
+		if (StrBufNum_tokens(SBuf, '-') > 1) {
+			extract_token(&ls->region[0], 
+				      ChrPtr(SBuf), 
+				      1, '-', 
+				      sizeof(ls->region));
 		}
 		else { /** no ara code? use lang code */
 			blen=strlen(&ls->lang[0]);
-			memcpy(&ls->region[0], ls->lang,blen);
-			ls->region[blen]='\0';
-		} /** area codes are uppercase */
-		blen=strlen(&ls->region[0]);
-		for (j=0; j<blen; j++)
-		{
-			int chars=toupper(ls->region[j]);
-			ls->region[j]=(char)chars;/** \todo ?! */
+			memcpy(&ls->region[0], ls->lang, blen);
+			ls->region[blen] = '\0';
 		}
-		sprintf(&lbuf[0],"%s_%s",&ls->lang[0],&ls->region[0]);
+
+		/** area codes are uppercase */
+		blen = strlen(&ls->region[0]);
+		for (j = 0; j < blen; j++)
+		{
+			int chars;
+			chars = toupper(ls->region[j]);
+			ls->region[j] = (char)chars;/** \todo ?! */
+		}
+		snprintf(&lbuf[0], 
+			 sizeof(lbuf), 
+			 "%s_%s", 
+			 &ls->lang[0], 
+			 &ls->region[0]);
 			
 		/** check if we have this lang */
-		ls->availability=1;
-		ls->selectedlang=-1;
-		for (j=0; j<nLocalesLoaded; j++) {
+		ls->availability = 1;
+		ls->selectedlang = -1;
+		for (j = 0; j < nLocalesLoaded; j++) {
 			int result;
 			/** match against the LANG part */
-			result=strcasecmp(&ls->lang[0], AvailLangLoaded[j]);
-			if ((result<0)&&(result<ls->availability)){
-				ls->availability=result;
-				ls->selectedlang=j;
+			result = strcasecmp(&ls->lang[0], AvailLangLoaded[j]);
+			if ((result < 0) && (result < ls->availability)){
+				ls->availability = result;
+				ls->selectedlang = j;
 			}
 			/** match against lang and locale */
-			if (0==strcasecmp(&lbuf[0], AvailLangLoaded[j])){
-				ls->availability=0;
-				ls->selectedlang=j;
-				j=nLocalesLoaded;
+			if (0 == strcasecmp(&lbuf[0], AvailLangLoaded[j])){
+				ls->availability = 0;
+				ls->selectedlang = j;
+				j = nLocalesLoaded;
 			}
 		}
         }
 	
-	prio=0;
-	av=-1000;
-	nBest=-1;
-	for (i=0; ((i<nParts)&&(i<SEARCH_LANG)); i++) {
-		ls=&wanted_locales[i];
-		if ((ls->availability<=0)&& 
-		   (av<ls->availability)&&
-		   (prio<ls->priority)&&
-		   (ls->selectedlang!=-1)) {
-			nBest=ls->selectedlang;
-			av=ls->availability;
-			prio=ls->priority;
+	prio = 0;
+	av = -1000;
+	nBest = -1;
+	for (i = 0; ((i < nParts) && (i<SEARCH_LANG)); i++) {
+		ls = &wanted_locales[i];
+		if ((ls->availability <= 0) && 
+		    (av < ls->availability) &&
+		    (prio < ls->priority) &&
+		    (ls->selectedlang != -1)) {
+			nBest = ls->selectedlang;
+			av = ls->availability;
+			prio = ls->priority;
 		}
 	}
 	if (nBest == -1) {
 		/** fall back to C */
 		nBest=0;
 	}
-	WC->selected_language=nBest;
+	sess->selected_language = nBest;
 	lprintf(9, "language found: %s\n", AvailLangLoaded[WC->selected_language]);
 	FreeStrBuf(&Buf);
 	FreeStrBuf(&SBuf);
@@ -175,24 +196,24 @@ void tmplput_offer_languages(StrBuf *Target, WCTemplputParams *TP)
 
 
 	if (nLocalesLoaded == 1) {
-		wprintf("<p>%s</p>", AvailLangLoaded[0]);
+		wc_printf("<p>%s</p>", AvailLangLoaded[0]);
 		return;
 	}
 
-	wprintf("<select name=\"language\" id=\"lname\" size=\"1\">\n");
+	wc_printf("<select name=\"language\" id=\"lname\" size=\"1\">\n");
 
 	for (i=0; i < nLocalesLoaded; ++i) {
 #ifndef HAVE_USELOCALE
 		if (strcmp(AvailLangLoaded[i], Lang) == 0)
 #endif
-		wprintf("<option %s value=%s>%s</option>\n",
+		wc_printf("<option %s value=%s>%s</option>\n",
 			((WC->selected_language == i) ? "selected" : ""),
 			AvailLangLoaded[i],
 			AvailLangLoaded[i]
 		);
 	}
 
-	wprintf("</select>\n");
+	wc_printf("</select>\n");
 }
 
 /**
@@ -202,7 +223,7 @@ void tmplput_offer_languages(StrBuf *Target, WCTemplputParams *TP)
 void set_selected_language(const char *lang) {
 #ifdef HAVE_USELOCALE
 	int i;
-	for (i=0; i<nLocalesLoaded; ++i) {
+	for (i = 0; i<nLocalesLoaded; ++i) {
 		if (!strcasecmp(lang, AvailLangLoaded[i])) {
 			WC->selected_language = i;
 		}
@@ -245,17 +266,28 @@ void stop_selected_language(void) {
  * \brief Create a locale_t for each available language
  */
 void initialize_locales(void) {
+	int nLocales;
 	int i;
 	char buf[32];
 	char *language = NULL;
-	
+	char *locale;
+
+
+	nLocales = 0; 
+	while (!IsEmptyStr(AvailLang[nLocales]))
+		nLocales++;
+
 	language = getenv("WEBCIT_LANG");
 	if ((language) && (!IsEmptyStr(language)) && (strcmp(language, "UNLIMITED") != 0)) {
 		lprintf(9, "Nailing locale to %s\n", language);
  	}
 	else language = NULL;
 
+	AvailLangLoaded = malloc (sizeof(char*) * nLocales);
+	memset(AvailLangLoaded, 0, sizeof(char*) * nLocales);
 #ifdef HAVE_USELOCALE
+	wc_locales = malloc (sizeof(locale_t) * nLocales);
+	memset(wc_locales,0, sizeof(locale_t) * nLocales);
 	/* create default locale */
 	Empty_Locale = newlocale(LC_ALL_MASK, NULL, NULL);
 #endif
@@ -263,7 +295,7 @@ void initialize_locales(void) {
 
 
 
-	for (i = 0; i < NUM_LANGS; ++i) {
+	for (i = 0; i < nLocales; ++i) {
 		if ((language != NULL) && (strcmp(AvailLang[i], language) != 0))
 			continue;
 		if (i == 0) {
@@ -279,20 +311,21 @@ void initialize_locales(void) {
 			(((i > 0) && (wc_locales[0] != NULL)) ? wc_locales[0] : Empty_Locale)
 		);
 		if (wc_locales[nLocalesLoaded] == NULL) {
-			lprintf(1, "Error configuring locale for %s: %s\n",
+			lprintf(1, "locale for "LOCALEDIR"locale/%s: %s; disabled\n",
 				buf,
 				strerror(errno)
 			);
 		}
 		else {
-			lprintf(3, "Configured available locale: %s\n", buf);
+			lprintf(3, "Found locale: %s\n", buf);
 			AvailLangLoaded[nLocalesLoaded] = AvailLang[i];
 			nLocalesLoaded++;
 		}
 #else
-		if (language != NULL) {
+		if ((language != NULL) && (strcmp(language, AvailLang[i]) == 0)) {
 			setenv("LANG", buf, 1);
 			AvailLangLoaded[nLocalesLoaded] = AvailLang[i];
+			setlocale(LC_MESSAGES, AvailLang[i]);
 			nLocalesLoaded++;
 		}
 		else if (nLocalesLoaded == 0) {
@@ -321,10 +354,20 @@ void initialize_locales(void) {
 
 #endif
 
+#ifdef ENABLE_NLS
+	locale = setlocale(LC_ALL, "");
+
+	lprintf(9, "Message catalog directory: %s\n", bindtextdomain("webcit", LOCALEDIR"/locale"));
+	lprintf(9, "Text domain: %s\n", textdomain("webcit"));
+	lprintf(9, "Text domain Charset: %s\n", bind_textdomain_codeset("webcit","UTF8"));
+
+#endif
 }
 
 
-void ShutdownLocale(void)
+void 
+ServerShutdownModule_GETTEXT
+(void)
 {
 #ifdef HAVE_USELOCALE
 	int i;
@@ -332,17 +375,19 @@ void ShutdownLocale(void)
 		if (Empty_Locale != wc_locales[i])
 			freelocale(wc_locales[i]);
 	}
+	free(wc_locales);
 #endif
+	free(AvailLangLoaded);
 }
 
 #else	/* ENABLE_NLS */
-const char *AvailLang[NUM_LANGS] = {
-	"C"};
+const char *AvailLang[] = {
+	"C", ""};
 
 /** \brief dummy for non NLS enabled systems */
 void tmplput_offer_languages(StrBuf *Target, WCTemplputParams *TP)
 {
-	wprintf("English (US)");
+	wc_printf("English (US)");
 }
 
 /** \brief dummy for non NLS enabled systems */
@@ -382,9 +427,52 @@ const char *get_selected_language(void) {
 #endif
 }
 
+
+void Header_HandleAcceptLanguage(StrBuf *Line, ParsedHttpHdrs *hdr)
+{
+	hdr->HR.browser_language = Line;
+}
+
 void 
 InitModule_GETTEXT
 (void)
 {
-	RegisterNamespace("LANG:SELECT", 0, 0, tmplput_offer_languages, CTX_NONE);
+	initialize_locales();
+	
+	RegisterHeaderHandler(HKEY("ACCEPT-LANGUAGE"), 
+			      Header_HandleAcceptLanguage);
+			      
+	RegisterNamespace("LANG:SELECT", 0, 0, 
+			  tmplput_offer_languages, NULL, CTX_NONE);
+}
+
+
+void
+SessionNewModule_GETTEXT
+(wcsession *sess)
+{
+#ifdef ENABLE_NLS
+	if (!sess->Hdr->HR.Static && 
+	    (sess->Hdr->HR.browser_language != NULL)) {
+		httplang_to_locale(sess->Hdr->HR.browser_language, sess);
+	}
+#endif
+}
+
+void
+SessionAttachModule_GETTEXT
+(wcsession *sess)
+{
+#ifdef ENABLE_NLS
+	go_selected_language();					/* set locale */
+#endif
+}
+
+void 
+SessionDestroyModule_GETTEXT
+(wcsession *sess)
+{
+#ifdef ENABLE_NLS
+	stop_selected_language();				/* unset locale */
+#endif
 }
