@@ -1,5 +1,5 @@
 /*
- * $Id: graphics.c 6999 2009-01-26 10:35:34Z matt $
+ * $Id: graphics.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * Handles HTTP upload of graphics files into the system.
  * \ingroup WebcitHttpServer
@@ -22,7 +22,7 @@ void display_graphics_upload(char *description, char *filename, char *uplurl)
 		display_main_menu();
 		return;
 	}
-	//output_headers(1, 1, 0, 0, 0, 0);
+	/*output_headers(1, 1, 0, 0, 0, 0); */
 
 	output_headers(1, 1, 1, 0, 0, 0);
 
@@ -34,30 +34,30 @@ void display_graphics_upload(char *description, char *filename, char *uplurl)
 
 	FreeStrBuf(&Buf);
 
-	wprintf("<form enctype=\"multipart/form-data\" action=\"%s\" "
+	wc_printf("<form enctype=\"multipart/form-data\" action=\"%s\" "
 		"method=\"post\" name=\"graphicsupload\">\n", uplurl);
 
-	wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
-	wprintf("<input type=\"hidden\" name=\"which_room\" value=\"");
+	wc_printf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
+	wc_printf("<input type=\"hidden\" name=\"which_room\" value=\"");
 	urlescputs(bstr("which_room"));
-	wprintf("\">\n");
+	wc_printf("\">\n");
 
-	wprintf(_("You can upload an image directly from your computer"));
-	wprintf("<br /><br />\n");
+	wc_printf(_("You can upload an image directly from your computer"));
+	wc_printf("<br /><br />\n");
 
-	wprintf(_("Please select a file to upload:"));
-	wprintf("<input type=\"file\" name=\"filename\" size=\"35\">\n");
+	wc_printf(_("Please select a file to upload:"));
+	wc_printf("<input type=\"file\" name=\"filename\" size=\"35\">\n");
 
-	wprintf("<div class=\"uploadpic\"><img src=\"image&name=%s\"></div>\n", filename);
+	wc_printf("<div class=\"uploadpic\"><img src=\"image?name=%s\"></div>\n", filename);
 
-	wprintf("<div class=\"buttons\">");
-	wprintf("<input type=\"submit\" name=\"upload_button\" value=\"%s\">\n", _("Upload"));
-	wprintf("&nbsp;");
-	wprintf("<input type=\"reset\" value=\"%s\">\n", _("Reset form"));
-	wprintf("&nbsp;");
-	wprintf("<input type=\"submit\" name=\"cancel_button\" value=\"%s\">\n", _("Cancel"));
-	wprintf("</div>\n");
-	wprintf("</form>\n");
+	wc_printf("<div class=\"buttons\">");
+	wc_printf("<input type=\"submit\" name=\"upload_button\" value=\"%s\">\n", _("Upload"));
+	wc_printf("&nbsp;");
+	wc_printf("<input type=\"reset\" value=\"%s\">\n", _("Reset form"));
+	wc_printf("&nbsp;");
+	wc_printf("<input type=\"submit\" name=\"cancel_button\" value=\"%s\">\n", _("Cancel"));
+	wc_printf("</div>\n");
+	wc_printf("</form>\n");
 
 	do_template("endbox", NULL);
 
@@ -67,11 +67,12 @@ void display_graphics_upload(char *description, char *filename, char *uplurl)
 void do_graphics_upload(char *filename)
 {
 	const char *MimeType;
+	wcsession *WCC = WC;
 	char buf[SIZ];
 	int bytes_remaining;
 	int pos = 0;
 	int thisblock;
-	bytes_remaining = WC->upload_length;
+	bytes_remaining = WCC->upload_length;
 
 	if (havebstr("cancel_button")) {
 		strcpy(WC->ImportantMessage,
@@ -80,20 +81,20 @@ void do_graphics_upload(char *filename)
 		return;
 	}
 
-	if (WC->upload_length == 0) {
+	if (WCC->upload_length == 0) {
 		strcpy(WC->ImportantMessage,
 			_("You didn't upload a file."));
 		display_main_menu();
 		return;
 	}
 	
-	MimeType = GuessMimeType(&WC->upload[0], bytes_remaining);
+	MimeType = GuessMimeType(ChrPtr(WCC->upload), bytes_remaining);
 	snprintf(buf, SIZ, "UIMG 1|%s|%s", MimeType, filename);
 	serv_puts(buf);
 
 	serv_getln(buf, sizeof buf);
 	if (buf[0] != '2') {
-		strcpy(WC->ImportantMessage, &buf[4]);
+		strcpy(WCC->ImportantMessage, &buf[4]);
 		display_main_menu();
 		return;
 	}
@@ -102,14 +103,14 @@ void do_graphics_upload(char *filename)
 		serv_printf("WRIT %d", thisblock);
 		serv_getln(buf, sizeof buf);
 		if (buf[0] != '7') {
-			strcpy(WC->ImportantMessage, &buf[4]);
+			strcpy(WCC->ImportantMessage, &buf[4]);
 			serv_puts("UCLS 0");
 			serv_getln(buf, sizeof buf);
 			display_main_menu();
 			return;
 		}
 		thisblock = extract_int(&buf[4], 0);
-		serv_write(&WC->upload[pos], thisblock);
+		serv_write(&ChrPtr(WCC->upload)[pos], thisblock);
 		pos = pos + thisblock;
 		bytes_remaining = bytes_remaining - thisblock;
 	}
@@ -156,16 +157,23 @@ void display_editgoodbyepic(void) {
 
 void display_editfloorpic(void) {
 	char buf[SIZ];
-	snprintf(buf, SIZ, "UIMG 0|_floorpic_|%s",
+	snprintf(buf, SIZ, "_floorpic_|%s",
 		 bstr("which_floor"));
 	display_graphics_upload(_("the icon for this floor"),
 				buf,
 				"editfloorpic");
 }
 
+void editroompic(void) {
+	char buf[SIZ];
+	snprintf(buf, SIZ, "_roompic_|%s",
+		 bstr("which_room"));
+	do_graphics_upload(buf);
+}
+
 void editfloorpic(void){
 	char buf[SIZ];
-	snprintf(buf, SIZ, "UIMG 1|_floorpic_|%s",
+	snprintf(buf, SIZ, "_floorpic_|%s",
 		 bstr("which_floor"));
 	do_graphics_upload(buf);
 }
@@ -174,13 +182,14 @@ void
 InitModule_GRAPHICS
 (void)
 {
-	WebcitAddUrlHandler(HKEY("display_editpic"), display_editpic, 0);
-	WebcitAddUrlHandler(HKEY("editpic"), editpic, 0);
-	WebcitAddUrlHandler(HKEY("display_editroompic"), display_editroompic, 0);
-	WebcitAddUrlHandler(HKEY("display_edithello"), display_edithello, 0);
-	WebcitAddUrlHandler(HKEY("edithellopic"), edithellopic, 0);
-	WebcitAddUrlHandler(HKEY("display_editgoodbyepic"), display_editgoodbyepic, 0);
-	WebcitAddUrlHandler(HKEY("editgoodbuyepic"), editgoodbuyepic, 0);
-	WebcitAddUrlHandler(HKEY("display_editfloorpic"), display_editfloorpic, 0);
-	WebcitAddUrlHandler(HKEY("editfloorpic"), editfloorpic, 0);
+	WebcitAddUrlHandler(HKEY("display_editpic"), "", 0, display_editpic, 0);
+	WebcitAddUrlHandler(HKEY("editpic"), "", 0, editpic, 0);
+	WebcitAddUrlHandler(HKEY("display_editroompic"), "", 0, display_editroompic, 0);
+	WebcitAddUrlHandler(HKEY("editroompic"), "", 0, editroompic, 0);
+	WebcitAddUrlHandler(HKEY("display_edithello"), "", 0, display_edithello, 0);
+	WebcitAddUrlHandler(HKEY("edithellopic"), "", 0, edithellopic, 0);
+	WebcitAddUrlHandler(HKEY("display_editgoodbuye"), "", 0, display_editgoodbyepic, 0);
+	WebcitAddUrlHandler(HKEY("editgoodbuyepic"), "", 0, editgoodbuyepic, 0);
+	WebcitAddUrlHandler(HKEY("display_editfloorpic"), "", 0, display_editfloorpic, 0);
+	WebcitAddUrlHandler(HKEY("editfloorpic"), "", 0, editfloorpic, 0);
 }
