@@ -1,5 +1,5 @@
 /*
- * $Id: groupdav_delete.c 7141 2009-02-20 22:14:54Z dothebart $
+ * $Id: groupdav_delete.c 8181 2009-12-31 10:30:11Z dothebart $
  *
  * Handles GroupDAV DELETE requests.
  *
@@ -13,35 +13,38 @@
 /*
  * The pathname is always going to be /groupdav/room_name/euid
  */
-void groupdav_delete(StrBuf *dav_pathname, char *dav_ifmatch) {
+void groupdav_delete(void) 
+{
+	wcsession *WCC = WC;
 	char dav_uid[SIZ];
 	long dav_msgnum = (-1);
 	char buf[SIZ];
 	int n = 0;
-
-	/* First, break off the "/groupdav/" prefix */
-	StrBufCutLeft(dav_pathname, 9);
-
+	StrBuf *dav_roomname = NewStrBuf();
+	
 	/* Now extract the message euid */
-	n = StrBufNum_tokens(dav_pathname, '/');
-	extract_token(dav_uid, ChrPtr(dav_pathname), n-1, '/', sizeof dav_uid);
-	StrBufRemove_token(dav_pathname, n-1, '/');
+	n = StrBufNum_tokens(WCC->Hdr->HR.ReqLine, '/');
+	extract_token(dav_uid, ChrPtr(WCC->Hdr->HR.ReqLine), n-1, '/', sizeof dav_uid);
+	StrBufExtract_token(dav_roomname, WCC->Hdr->HR.ReqLine, 0, '/');
 
-	/* What's left is the room name.  Remove trailing slashes. */
-	//len = StrLength(dav_pathname);
-	//if ((len > 0) && (ChrPtr(dav_pathname)[len-1] == '/')) {
-	//	StrBufCutRight(dav_pathname, 1);
+	///* What's left is the room name.  Remove trailing slashes. */
+	//len = StrLength(WCC->Hdr->HR.ReqLine);
+	//if ((len > 0) && (ChrPtr(WCC->Hdr->HR.ReqLinee)[len-1] == '/')) {
+	//	StrBufCutRight(WCC->Hdr->HR.ReqLine, 1);
 	//}
-	StrBufCutLeft(dav_pathname, 1);
+	//StrBufCutLeft(WCC->Hdr->HR.ReqLine, 1);
 
 	/* Go to the correct room. */
-	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_pathname))) {
-		gotoroom(dav_pathname);
+	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_roomname))) {
+		gotoroom(dav_roomname);
 	}
-	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_pathname))) {
+	if (strcasecmp(ChrPtr(WC->wc_roomname), ChrPtr(dav_roomname))) {
 		hprintf("HTTP/1.1 404 not found\r\n");
 		groupdav_common_headers();
 		hprintf("Content-Length: 0\r\n\r\n");
+		begin_burst();
+		end_burst();
+		FreeStrBuf(&dav_roomname);
 		return;
 	}
 
@@ -54,6 +57,9 @@ void groupdav_delete(StrBuf *dav_pathname, char *dav_ifmatch) {
 		hprintf("HTTP/1.1 404 Not Found\r\n");
 		groupdav_common_headers();
 		hprintf("Content-Length: 0\r\n\r\n");
+		begin_burst();
+		end_burst();
+		FreeStrBuf(&dav_roomname);
 		return;
 	}
 
@@ -61,11 +67,14 @@ void groupdav_delete(StrBuf *dav_pathname, char *dav_ifmatch) {
 	 * It's there ... check the ETag and make sure it matches
 	 * the message number.
 	 */
-	if (!IsEmptyStr(dav_ifmatch)) {
-		if (atol(dav_ifmatch) != dav_msgnum) {
+	if (StrLength(WCC->Hdr->HR.dav_ifmatch) > 0) {
+		if (StrTol(WCC->Hdr->HR.dav_ifmatch) != dav_msgnum) {
 			hprintf("HTTP/1.1 412 Precondition Failed\r\n");
 			groupdav_common_headers();
 			hprintf("Content-Length: 0\r\n\r\n");
+			begin_burst();
+			end_burst();
+			FreeStrBuf(&dav_roomname);
 			return;
 		}
 	}
@@ -79,11 +88,16 @@ void groupdav_delete(StrBuf *dav_pathname, char *dav_ifmatch) {
 		hprintf("HTTP/1.1 204 No Content\r\n");	/* success */
 		groupdav_common_headers();
 		hprintf("Content-Length: 0\r\n\r\n");
+		begin_burst();
+		end_burst();
 	}
 	else {
 		hprintf("HTTP/1.1 403 Forbidden\r\n");	/* access denied */
 		groupdav_common_headers();
 		hprintf("Content-Length: 0\r\n\r\n");
+		begin_burst();
+		end_burst();
 	}
+	FreeStrBuf(&dav_roomname);
 	return;
 }
