@@ -1,5 +1,5 @@
 /*
- * $Id: http_datestring.c 6912 2009-01-08 23:11:56Z dothebart $
+ * $Id: http_datestring.c 7930 2009-10-20 16:25:13Z dothebart $
  */
 /**
  * \defgroup HTTPDateTime Function to generate HTTP-compliant textual time/date stamp
@@ -66,9 +66,18 @@ void http_datestring(char *buf, size_t n, time_t xtime) {
 
 void tmplput_nowstr(StrBuf *Target, WCTemplputParams *TP)
 {
+	char buf[64];
+	long bufused;
 	time_t now;
+	
 	now = time(NULL);
-	StrEscAppend(Target, NULL, asctime(localtime(&now)), 0, 0);
+	asctime_r(localtime(&now), buf);
+	bufused = strlen(buf);
+	if ((bufused > 0) && (buf[bufused - 1] == '\n')) {
+		buf[bufused - 1] = '\0';
+		bufused --;
+	}
+	StrEscAppend(Target, NULL, buf, 0, 0);
 }
 void tmplput_nowno(StrBuf *Target, WCTemplputParams *TP)
 {
@@ -81,8 +90,8 @@ void
 InitModule_DATE
 (void)
 {
-	RegisterNamespace("DATE:NOW:STR", 0, 0, tmplput_nowstr, CTX_NONE);
-	RegisterNamespace("DATE:NOW:NO", 0, 0, tmplput_nowno, CTX_NONE);
+	RegisterNamespace("DATE:NOW:STR", 0, 0, tmplput_nowstr, NULL, CTX_NONE);
+	RegisterNamespace("DATE:NOW:NO", 0, 0, tmplput_nowno, NULL, CTX_NONE);
 }
 
 /*@}*/
