@@ -1,8 +1,8 @@
-/*----------------------------------------------------------------------------*/
 #include "webcit.h"
 #include "webserver.h"
+#include "groupdav.h"
 
-/**
+/*
  * message index functions
  */
 
@@ -61,14 +61,18 @@ void RegisterMsgHdr(const char *HeaderName, long HdrNLen, ExamineMsgHeaderFunc e
 	Put(MsgHeaderHandler, HeaderName, HdrNLen, ev, NULL);
 }
 
-void RegisterMimeRenderer(const char *HeaderName, long HdrNLen, RenderMimeFunc MimeRenderer)
+void RegisterMimeRenderer(const char *HeaderName, long HdrNLen, 
+			  RenderMimeFunc MimeRenderer,
+			  int InlineRenderable,
+			  int Priority)
 {
 	RenderMimeFuncStruct *f;
 
 	f = (RenderMimeFuncStruct*) malloc(sizeof(RenderMimeFuncStruct));
 	f->f = MimeRenderer;
 	Put(MimeRenderHandler, HeaderName, HdrNLen, f, NULL);
-	
+	if (InlineRenderable)
+		RegisterEmbeddableMimeType(HeaderName, HdrNLen, 10000 - Priority);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -462,16 +466,21 @@ void tmplput_MAIL_SUMM_DATE_NO(StrBuf *Target, WCTemplputParams *TP)
 
 void render_MAIL(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
 {
-	Mime->Data = NewStrBufPlain(NULL, Mime->length);
-	read_message(Mime->Data, HKEY("view_submessage"), Mime->msgnum, 0, Mime->PartNum);
+	const StrBuf *TemplateMime;
+
+	if (Mime->Data == NULL) 
+		Mime->Data = NewStrBufPlain(NULL, Mime->length);
+	else 
+		FlushStrBuf(Mime->Data);
+	read_message(Mime->Data, HKEY("view_submessage"), Mime->msgnum, Mime->PartNum, &TemplateMime);
 /*
 	if ( (!IsEmptyStr(mime_submessages)) && (!section[0]) ) {
 		for (i=0; i<num_tokens(mime_submessages, '|'); ++i) {
 			extract_token(buf, mime_submessages, i, '|', sizeof buf);
 			/ ** use printable_view to suppress buttons * /
-			wprintf("<blockquote>");
+			wc_printf("<blockquote>");
 			read_message(Mime->msgnum, 1, ChrPtr(Mime->Section));
-			wprintf("</blockquote>");
+			wc_printf("</blockquote>");
 		}
 	}
 */
@@ -489,19 +498,52 @@ void render_MIME_VCard(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundC
 			|| (!strcasecmp(&(ChrPtr(WCC->wc_roomname)[11]), USERCONFIGROOM))
 			|| (WC->wc_view == VIEW_ADDRESSBOOK)
 			) {
-			StrBufAppendPrintf(Buf, "<a href=\"edit_vcard?msgnum=%ld&partnum=%s\">",
+			StrBufAppendPrintf(Buf, "<a href=\"edit_vcard?msgnum=%ld?partnum=%s\">",
 				Mime->msgnum, ChrPtr(Mime->PartNum));
 			StrBufAppendPrintf(Buf, "[%s]</a>", _("edit"));
 		}
 
 		/* In all cases, display the full card */
-		display_vcard(Buf, ChrPtr(Mime->Data), 0, 1, NULL, Mime->msgnum);
+		display_vcard(Buf, Mime->Data, 0, 1, NULL, Mime->msgnum);
 		FreeStrBuf(&Mime->Data);
 		Mime->Data = Buf;
 	}
 
 }
 
+void render_MIME_VNote(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+{
+	MimeLoadData(Mime);
+	if (StrLength(Mime->Data) > 0) {
+		struct vnote *v;
+		StrBuf *Buf;
+		char *vcard;
+
+		Buf = NewStrBuf();
+		vcard = SmashStrBuf(&Mime->Data);
+		v = vnote_new_from_str(vcard);
+		free (vcard);
+		if (v) {
+			WCTemplputParams TP;
+			
+			memset(&TP, 0, sizeof(WCTemplputParams));
+			TP.Filter.ContextType = CTX_VNOTE;
+			TP.Context = v;
+			DoTemplate(HKEY("mail_vnoteitem"),
+				   Buf, &TP);
+			
+			vnote_free(v);
+			Mime->Data = Buf;
+		}
+		else {
+			if (Mime->Data == NULL)
+				Mime->Data = NewStrBuf();
+			else
+				FlushStrBuf(Mime->Data);
+		}
+	}
+}
+
 void render_MIME_ICS(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
 {
 	if (StrLength(Mime->Data) == 0) {
@@ -643,7 +685,10 @@ int Conditional_MAIL_SUMM_H_NODE(StrBuf *Target, WCTemplputParams *TP)
 
 void examine_text(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 {
-	Msg->MsgBody->Data = NewStrBuf();
+	if (Msg->MsgBody->Data == NULL)
+		Msg->MsgBody->Data = NewStrBufPlain(NULL, SIZ);
+	else
+		FlushStrBuf(Msg->MsgBody->Data);
 }
 
 void examine_msg4_partnum(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
@@ -741,21 +786,47 @@ int Conditional_MAIL_MIME_ATTACH(StrBuf *Target, WCTemplputParams *TP)
 	return GetCount(Msg->AllAttach) > 0;
 }
 
-
-
-/*----------------------------------------------------------------------------*/
 void tmplput_QUOTED_MAIL_BODY(StrBuf *Target, WCTemplputParams *TP)
 {
+	const StrBuf *Mime;
         long MsgNum;
 	StrBuf *Buf;
 
 	MsgNum = LBstr(TKEY(0));
 	Buf = NewStrBuf();
-	read_message(Buf, HKEY("view_message_replyquote"), MsgNum, 0, NULL);
+	read_message(Buf, HKEY("view_message_replyquote"), MsgNum, NULL, &Mime);
 	StrBufAppendTemplate(Target, TP, Buf, 1);
 	FreeStrBuf(&Buf);
 }
 
+void tmplput_EDIT_MAIL_BODY(StrBuf *Target, WCTemplputParams *TP)
+{
+	const StrBuf *Mime;
+        long MsgNum;
+	StrBuf *Buf;
+
+	MsgNum = LBstr(TKEY(0));
+	Buf = NewStrBuf();
+	read_message(Buf, HKEY("view_message_edit"), MsgNum, NULL, &Mime);
+	StrBufAppendTemplate(Target, TP, Buf, 1);
+	FreeStrBuf(&Buf);
+}
+
+void tmplput_EDIT_WIKI_BODY(StrBuf *Target, WCTemplputParams *TP)
+{
+	const StrBuf *Mime;
+        long msgnum;
+	StrBuf *Buf;
+
+	msgnum = locate_message_by_uid(BSTR("page"));
+	if (msgnum >= 0L) {
+		Buf = NewStrBuf();
+		read_message(Buf, HKEY("view_message_wikiedit"), msgnum, NULL, &Mime);
+		StrBufAppendTemplate(Target, TP, Buf, 1);
+		FreeStrBuf(&Buf);
+	}
+}
+
 void tmplput_MAIL_BODY(StrBuf *Target, WCTemplputParams *TP)
 {
 	message_summary *Msg = (message_summary*) CTX;
@@ -774,7 +845,6 @@ void render_MAIL_variformat(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 
 void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
 {
-	StrBuf *cs = NULL;
 	const char *ptr, *pte;
 	const char *BufPtr = NULL;
 	StrBuf *Line;
@@ -782,12 +852,13 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 	StrBuf *Line2;
 	StrBuf *Target;
 
-	int ConvertIt = 1;
 	int bn = 0;
 	int bq = 0;
-	int i, n, done = 0;
+	int i;
 	long len;
 #ifdef HAVE_ICONV
+	StrBuf *cs = NULL;
+	int ConvertIt = 1;
 	iconv_t ic = (iconv_t)(-1) ;
 #endif
 
@@ -796,11 +867,6 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 		MimeLoadData(Mime);
 	}
 
-	/* Boring old 80-column fixed format text gets handled this way... */
-	if ((strcasecmp(ChrPtr(Mime->Charset), "us-ascii") == 0) &&
-	    (strcasecmp(ChrPtr(Mime->Charset), "UTF-8") == 0))
-		ConvertIt = 0;
-
 #ifdef HAVE_ICONV
 	if (ConvertIt) {
 		if (StrLength(Mime->Charset) != 0)
@@ -812,6 +878,12 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 		if (cs == NULL) {
 			ConvertIt = 0;
 		}
+		else if (!strcasecmp(ChrPtr(cs), "utf-8")) {
+			ConvertIt = 0;
+		}
+		else if (!strcasecmp(ChrPtr(cs), "us-ascii")) {
+			ConvertIt = 0;
+		}
 		else {
 			ctdl_iconv_open("UTF-8", ChrPtr(cs), &ic);
 			if (ic == (iconv_t)(-1) ) {
@@ -821,52 +893,55 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 		}
 	}
 #endif
-	Line = NewStrBuf();
-	Line1 = NewStrBuf();
-	Line2 = NewStrBuf();
+	Line = NewStrBufPlain(NULL, SIZ);
+	Line1 = NewStrBufPlain(NULL, SIZ);
+	Line2 = NewStrBufPlain(NULL, SIZ);
 	Target = NewStrBufPlain(NULL, StrLength(Mime->Data));
 
-	while ((n = StrBufSipLine(Line, Mime->Data, &BufPtr), n >= 0) && !done)
-	{
-		done = n == 0;
-		bq = 0;
-		i = 0;
-		ptr = ChrPtr(Line);
-		len = StrLength(Line);
-		pte = ptr + len;
-		
-		while ((ptr < pte) &&
-		       ((*ptr == '>') ||
-			isspace(*ptr)))
+	if (StrLength(Mime->Data) > 0) 
+		do 
 		{
-			if (*ptr == '>')
-				bq++;
-			ptr ++;
-			i++;
-		}
-		if (i > 0) StrBufCutLeft(Line, i);
+			StrBufSipLine(Line, Mime->Data, &BufPtr);
+			bq = 0;
+			i = 0;
+			ptr = ChrPtr(Line);
+			len = StrLength(Line);
+			pte = ptr + len;
 		
-		if (StrLength(Line) == 0) {
-			StrBufAppendBufPlain(Target, HKEY("<tt></tt><br />\n"), 0);
-			continue;
+			while ((ptr < pte) &&
+			       ((*ptr == '>') ||
+				isspace(*ptr)))
+			{
+				if (*ptr == '>')
+					bq++;
+				ptr ++;
+				i++;
+			}
+			if (i > 0) StrBufCutLeft(Line, i);
+		
+			if (StrLength(Line) == 0) {
+				StrBufAppendBufPlain(Target, HKEY("<tt></tt><br />\n"), 0);
+				continue;
+			}
+
+			for (i = bn; i < bq; i++)				
+				StrBufAppendBufPlain(Target, HKEY("<blockquote>"), 0);
+			for (i = bq; i < bn; i++)				
+				StrBufAppendBufPlain(Target, HKEY("</blockquote>"), 0);
+#ifdef HAVE_ICONV
+			if (ConvertIt) {
+				StrBufConvert(Line, Line1, &ic);
+			}
+#endif
+			StrBufAppendBufPlain(Target, HKEY("<tt>"), 0);
+			UrlizeText(Line1, Line, Line2);
+
+			StrEscAppend(Target, Line1, NULL, 0, 0);
+			StrBufAppendBufPlain(Target, HKEY("</tt><br />\n"), 0);
+			bn = bq;
 		}
-
-		for (i = bn; i < bq; i++)				
-			StrBufAppendBufPlain(Target, HKEY("<blockquote>"), 0);
-		for (i = bq; i < bn; i++)				
-			StrBufAppendBufPlain(Target, HKEY("</blockquote>"), 0);
-
-		if (ConvertIt == 1) {
-			StrBufConvert(Line, Line1, &ic);
-		}
-
-		StrBufAppendBufPlain(Target, HKEY("<tt>"), 0);
-		UrlizeText(Line1, Line, Line2);
-
-		StrEscAppend(Target, Line1, NULL, 0, 0);
-		StrBufAppendBufPlain(Target, HKEY("</tt><br />\n"), 0);
-		bn = bq;
-	}
+	while ((BufPtr != StrBufNOTNULL) &&
+	       (BufPtr != NULL));
 
 	for (i = 0; i < bn; i++)				
 		StrBufAppendBufPlain(Target, HKEY("</blockquote>"), 0);
@@ -917,10 +992,6 @@ void render_MAIL_UNKNOWN(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *Foun
 }
 
 
-
-
-
-
 HashList *iterate_get_mime_All(StrBuf *Target, WCTemplputParams *TP)
 {
 	message_summary *Msg = (message_summary*) CTX;
@@ -1004,8 +1075,9 @@ void tmplput_MIME_LoadData(StrBuf *Target, WCTemplputParams *TP)
 	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
 	wc_mime_attachment *att;
 	
-	if ( (!strcasecmp(ChrPtr(mime->Disposition), "inline"))||
-	     (!strcasecmp(ChrPtr(mime->Disposition), "attachment")) ) 
+	if (( (!strcasecmp(ChrPtr(mime->Disposition), "inline"))||
+	      (!strcasecmp(ChrPtr(mime->Disposition), "attachment"))) && 
+	    (strcasecmp(ChrPtr(mime->ContentType), "application/ms-tnef")!=0))
 	{
  		
  		int n;
@@ -1052,6 +1124,11 @@ void servcmd_readfwd(char *buf, long bufsize)
 	snprintf(buf, bufsize, "MSGS ALL");
 }
 
+void servcmd_readgt(char *buf, long bufsize)
+{
+	snprintf(buf, bufsize, "MSGS GT|%s", bstr("gt"));
+}
+
 void servcmd_readnew(char *buf, long bufsize)
 {
 	snprintf(buf, bufsize, "MSGS NEW");
@@ -1068,32 +1145,81 @@ readloop_struct rlid[] = {
 	{ {HKEY("headers")},   servcmd_headers},
 	{ {HKEY("readfwd")},   servcmd_readfwd},
 	{ {HKEY("readnew")},   servcmd_readnew},
-	{ {HKEY("readold")},   servcmd_readold}
+	{ {HKEY("readold")},   servcmd_readold},
+	{ {HKEY("readgt")},    servcmd_readgt}
 };
 
-
-
-void SetAccessCommand(long Oper)
-{
-	wcsession *WCC = WC;	
-
-	if (WCC->UrlFragment1 != NULL ) {
-		FlushStrBuf(WCC->UrlFragment1);
-		StrBufAppendBufPlain(WCC->UrlFragment1, 
-				     rlid[Oper].name.Key, rlid[Oper].name.len, 0);
-	}
-	else 
-		WCC->UrlFragment1 = NewStrBufPlain(rlid[Oper].name.Key, rlid[Oper].name.len);
+/* Spit out the new summary view. This is basically a static page, so clients can cache the layout, all the dirty work is javascript :) */
+void new_summary_view(void) {
+	DoTemplate(HKEY("msg_listview"),NULL,&NoCtx);
 }
-		
 
 
+int mailview_GetParamsGetServerCall(SharedMessageStatus *Stat, 
+				    void **ViewSpecific, 
+				    long oper, 
+				    char *cmd, 
+				    long len)
+{
+	if (!WC->is_ajax) {
+		new_summary_view();
+		return 200;
+	} else {
+		Stat->defaultsortorder = 2;
+		Stat->sortit = 1;
+		Stat->load_seen = 1;
+		/* Generally using maxmsgs|startmsg is not required
+		   in mailbox view, but we have a 'safemode' for clients
+		   (*cough* Exploder) that simply can't handle too many */
+		if (havebstr("maxmsgs"))  Stat->maxmsgs  = ibstr("maxmsgs");
+		else                      Stat->maxmsgs  = 9999999;
+		if (havebstr("startmsg")) Stat->startmsg = lbstr("startmsg");
+		snprintf(cmd, len, "MSGS %s|%s||1",
+			 (oper == do_search) ? "SEARCH" : "ALL",
+			 (oper == do_search) ? bstr("query") : ""
+			);
+	}
+	return 200;
+}
+
+int mailview_RenderView_or_Tail(SharedMessageStatus *Stat, 
+				void **ViewSpecific, 
+				long oper)
+{
+	WCTemplputParams SubTP;
+
+	if (WC->is_ajax)
+		DoTemplate(HKEY("mailsummary_json"),NULL, &SubTP);
+	
+	return 0;
+}
+
+int mailview_Cleanup(void **ViewSpecific)
+{
+	/* Note: wDumpContent() will output one additional </div> tag. */
+	/* We ought to move this out into template */
+	if (WC->is_ajax)
+		end_burst();
+	else
+		wDumpContent(1);
+
+	return 0;
+}
+
 
 
 void 
 InitModule_MSGRENDERERS
 (void)
 {
+	RegisterReadLoopHandlerset(
+		VIEW_MAILBOX,
+		mailview_GetParamsGetServerCall,
+		NULL, /// TODO: is this right?
+		NULL, //// ""
+		mailview_RenderView_or_Tail,
+		mailview_Cleanup);
+
 	RegisterSortFunc(HKEY("date"), 
 			 NULL, 0,
 			 summcmp_date,
@@ -1113,29 +1239,31 @@ InitModule_MSGRENDERERS
 			 groupchange_sender,
 			 CTX_MAILSUM);
 
-	RegisterNamespace("SUMM:COUNT", 0, 0, tmplput_SUMM_COUNT, CTX_NONE);
+	RegisterNamespace("SUMM:COUNT", 0, 0, tmplput_SUMM_COUNT, NULL, CTX_NONE);
 	/* iterate over all known mails in WC->summ */
 	RegisterIterator("MAIL:SUMM:MSGS", 0, NULL, iterate_get_mailsumm_All,
 			 NULL,NULL, CTX_MAILSUM, CTX_NONE, IT_NOFLAG);
 
-	RegisterNamespace("MAIL:SUMM:DATEBRIEF", 0, 0, tmplput_MAIL_SUMM_DATE_BRIEF, CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:DATEFULL", 0, 0, tmplput_MAIL_SUMM_DATE_FULL, CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:DATENO",  0, 0, tmplput_MAIL_SUMM_DATE_NO,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:N",       0, 0, tmplput_MAIL_SUMM_N,        CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:FROM",    0, 2, tmplput_MAIL_SUMM_FROM,     CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:TO",      0, 2, tmplput_MAIL_SUMM_TO,       CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:SUBJECT", 0, 4, tmplput_MAIL_SUMM_SUBJECT,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:NTATACH", 0, 0, tmplput_MAIL_SUMM_NATTACH,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:CCCC", 0, 2, tmplput_MAIL_SUMM_CCCC,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:H_NODE", 0, 2, tmplput_MAIL_SUMM_H_NODE,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:ALLRCPT", 0, 2, tmplput_MAIL_SUMM_ALLRCPT,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:ORGROOM", 0, 2, tmplput_MAIL_SUMM_ORGROOM,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:RFCA", 0, 2, tmplput_MAIL_SUMM_RFCA,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:OTHERNODE", 2, 0, tmplput_MAIL_SUMM_OTHERNODE,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:REFIDS", 0, 1, tmplput_MAIL_SUMM_REFIDS,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:SUMM:INREPLYTO", 0, 2, tmplput_MAIL_SUMM_INREPLYTO,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:BODY", 0, 2, tmplput_MAIL_BODY,  CTX_MAILSUM);
-	RegisterNamespace("MAIL:QUOTETEXT", 1, 2, tmplput_QUOTED_MAIL_BODY,  CTX_NONE);
+	RegisterNamespace("MAIL:SUMM:DATEBRIEF", 0, 0, tmplput_MAIL_SUMM_DATE_BRIEF, NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:DATEFULL", 0, 0, tmplput_MAIL_SUMM_DATE_FULL, NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:DATENO",  0, 0, tmplput_MAIL_SUMM_DATE_NO,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:N",       0, 0, tmplput_MAIL_SUMM_N,        NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:FROM",    0, 2, tmplput_MAIL_SUMM_FROM,     NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:TO",      0, 2, tmplput_MAIL_SUMM_TO,       NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:SUBJECT", 0, 4, tmplput_MAIL_SUMM_SUBJECT,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:NTATACH", 0, 0, tmplput_MAIL_SUMM_NATTACH,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:CCCC", 0, 2, tmplput_MAIL_SUMM_CCCC, NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:H_NODE", 0, 2, tmplput_MAIL_SUMM_H_NODE,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:ALLRCPT", 0, 2, tmplput_MAIL_SUMM_ALLRCPT,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:ORGROOM", 0, 2, tmplput_MAIL_SUMM_ORGROOM,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:RFCA", 0, 2, tmplput_MAIL_SUMM_RFCA, NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:OTHERNODE", 2, 0, tmplput_MAIL_SUMM_OTHERNODE,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:REFIDS", 0, 1, tmplput_MAIL_SUMM_REFIDS,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:SUMM:INREPLYTO", 0, 2, tmplput_MAIL_SUMM_INREPLYTO,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:BODY", 0, 2, tmplput_MAIL_BODY,  NULL, CTX_MAILSUM);
+	RegisterNamespace("MAIL:QUOTETEXT", 1, 2, tmplput_QUOTED_MAIL_BODY,  NULL, CTX_NONE);
+	RegisterNamespace("MAIL:EDITTEXT", 1, 2, tmplput_EDIT_MAIL_BODY,  NULL, CTX_NONE);
+	RegisterNamespace("MAIL:EDITWIKI", 1, 2, tmplput_EDIT_WIKI_BODY,  NULL, CTX_NONE);
 	RegisterConditional(HKEY("COND:MAIL:SUMM:RFCA"), 0, Conditional_MAIL_SUMM_RFCA,  CTX_MAILSUM);
 	RegisterConditional(HKEY("COND:MAIL:SUMM:CCCC"), 0, Conditional_MAIL_SUMM_CCCC,  CTX_MAILSUM);
 	RegisterConditional(HKEY("COND:MAIL:SUMM:UNREAD"), 0, Conditional_MAIL_SUMM_UNREAD, CTX_MAILSUM);
@@ -1161,33 +1289,36 @@ InitModule_MSGRENDERERS
 			 NULL, NULL, CTX_MIME_ATACH, CTX_MAILSUM, IT_NOFLAG);
 
 	/* Parts of a mime attachent */
-	RegisterNamespace("MAIL:MIME:NAME", 0, 2, tmplput_MIME_Name, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:FILENAME", 0, 2, tmplput_MIME_FileName, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:PARTNUM", 0, 2, tmplput_MIME_PartNum, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:MSGNUM", 0, 2, tmplput_MIME_MsgNum, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:DISPOSITION", 0, 2, tmplput_MIME_Disposition, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:CONTENTTYPE", 0, 2, tmplput_MIME_ContentType, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:CHARSET", 0, 2, tmplput_MIME_Charset, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:LENGTH", 0, 2, tmplput_MIME_Length, CTX_MIME_ATACH);
-	RegisterNamespace("MAIL:MIME:DATA", 0, 2, tmplput_MIME_Data, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:NAME", 0, 2, tmplput_MIME_Name, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:FILENAME", 0, 2, tmplput_MIME_FileName, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:PARTNUM", 0, 2, tmplput_MIME_PartNum, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:MSGNUM", 0, 2, tmplput_MIME_MsgNum, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:DISPOSITION", 0, 2, tmplput_MIME_Disposition, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:CONTENTTYPE", 0, 2, tmplput_MIME_ContentType, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:CHARSET", 0, 2, tmplput_MIME_Charset, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:LENGTH", 0, 2, tmplput_MIME_Length, NULL, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:DATA", 0, 2, tmplput_MIME_Data, NULL, CTX_MIME_ATACH);
 	/* load the actual attachment into WC->attachments; no output!!! */
-	RegisterNamespace("MAIL:MIME:LOADDATA", 0, 0, tmplput_MIME_LoadData, CTX_MIME_ATACH);
+	RegisterNamespace("MAIL:MIME:LOADDATA", 0, 0, tmplput_MIME_LoadData, NULL, CTX_MIME_ATACH);
 
 	/* iterate the WC->attachments; use the above tokens for their contents */
 	RegisterIterator("MSG:ATTACHNAMES", 0, NULL, iterate_get_registered_Attachments, 
 			 NULL, NULL, CTX_MIME_ATACH, CTX_NONE, IT_NOFLAG);
 
 	/* mime renderers translate an attachment into webcit viewable html text */
-	RegisterMimeRenderer(HKEY("message/rfc822"), render_MAIL);
-	RegisterMimeRenderer(HKEY("text/x-vcard"), render_MIME_VCard);
-	RegisterMimeRenderer(HKEY("text/vcard"), render_MIME_VCard);
-	RegisterMimeRenderer(HKEY("text/calendar"), render_MIME_ICS);
-	RegisterMimeRenderer(HKEY("application/ics"), render_MIME_ICS);
-	RegisterMimeRenderer(HKEY("text/x-citadel-variformat"), render_MAIL_variformat);
-	RegisterMimeRenderer(HKEY("text/plain"), render_MAIL_text_plain);
-	RegisterMimeRenderer(HKEY("text"), render_MAIL_text_plain);
-	RegisterMimeRenderer(HKEY("text/html"), render_MAIL_html);
-	RegisterMimeRenderer(HKEY(""), render_MAIL_UNKNOWN);
+	RegisterMimeRenderer(HKEY("message/rfc822"), render_MAIL, 1, 150);
+	RegisterMimeRenderer(HKEY("text/vnote"), render_MIME_VNote, 1, 300);
+	RegisterMimeRenderer(HKEY("text/x-vcard"), render_MIME_VCard, 1, 201);
+	RegisterMimeRenderer(HKEY("text/vcard"), render_MIME_VCard, 1, 200);
+	RegisterMimeRenderer(HKEY("text/calendar"), render_MIME_ICS, 1, 501);
+	RegisterMimeRenderer(HKEY("application/ics"), render_MIME_ICS, 1, 500);
+	RegisterMimeRenderer(HKEY("text/x-citadel-variformat"), render_MAIL_variformat, 1, 2);
+	RegisterMimeRenderer(HKEY("text/plain"), render_MAIL_text_plain, 1, 3);
+	RegisterMimeRenderer(HKEY("text"), render_MAIL_text_plain, 1, 1);
+	RegisterMimeRenderer(HKEY("text/html"), render_MAIL_html, 1, 100);
+	RegisterMimeRenderer(HKEY(""), render_MAIL_UNKNOWN, 0, 0);
+	/* and finalize the anouncement to the server... */
+	CreateMimeStr();
 
 	/* these headers are citserver replies to MSG4 and friends. one evaluator for each */
 	RegisterMsgHdr(HKEY("nhdr"), examine_nhdr, 0);
@@ -1217,3 +1348,30 @@ InitModule_MSGRENDERERS
 	RegisterMsgHdr(HKEY("suff"), examine_suff, 0);
 	RegisterMsgHdr(HKEY("path"), examine_path, 0);
 }
+
+void 
+ServerStartModule_MSGRENDERERS
+(void)
+{
+	MsgHeaderHandler = NewHash(1, NULL);
+	MimeRenderHandler = NewHash(1, NULL);
+	ReadLoopHandler = NewHash(1, NULL);
+}
+
+void 
+ServerShutdownModule_MSGRENDERERS
+(void)
+{
+	DeleteHash(&MsgHeaderHandler);
+	DeleteHash(&MimeRenderHandler);
+	DeleteHash(&ReadLoopHandler);
+}
+
+
+
+void 
+SessionDestroyModule_MSGRENDERERS
+(wcsession *sess)
+{
+	DeleteHash(&sess->attachments);
+}
