@@ -1,5 +1,5 @@
 /* 
- * $Id: netconf.c 6952 2009-01-14 23:46:21Z dothebart $
+ * $Id: netconf.c 7928 2009-10-20 12:52:58Z dothebart $
  */
 /**
  * \defgroup NetShareConf Functions which handle network and sharing configuration.
@@ -298,18 +298,18 @@ void
 InitModule_NETCONF
 (void)
 {
-	WebcitAddUrlHandler(HKEY("display_edit_node"), display_edit_node, 0);
+	WebcitAddUrlHandler(HKEY("display_edit_node"), "", 0, display_edit_node, 0);
 
-	WebcitAddUrlHandler(HKEY("edit_node"), edit_node, 0);
-	WebcitAddUrlHandler(HKEY("display_netconf"), display_netconf, 0);
-	WebcitAddUrlHandler(HKEY("display_confirm_delete_node"), display_confirm_delete_node, 0);
-	WebcitAddUrlHandler(HKEY("delete_node"), delete_node, 0);
+	WebcitAddUrlHandler(HKEY("edit_node"), "", 0, edit_node, 0);
+	WebcitAddUrlHandler(HKEY("display_netconf"), "", 0, display_netconf, 0);
+	WebcitAddUrlHandler(HKEY("display_confirm_delete_node"), "", 0, display_confirm_delete_node, 0);
+	WebcitAddUrlHandler(HKEY("delete_node"), "", 0, delete_node, 0);
 
                                                                                           
-        RegisterNamespace("CFG:IGNET:NODE", 0, 1, tmplput_NodeName, CTX_NODECONF);
-        RegisterNamespace("CFG:IGNET:SECRET", 0, 1, tmplput_Secret, CTX_NODECONF);
-        RegisterNamespace("CFG:IGNET:HOST", 0, 1, tmplput_Host, CTX_NODECONF);
-        RegisterNamespace("CFG:IGNET:PORT", 0, 1, tmplput_Port, CTX_NODECONF);
+        RegisterNamespace("CFG:IGNET:NODE", 0, 1, tmplput_NodeName, NULL, CTX_NODECONF);
+        RegisterNamespace("CFG:IGNET:SECRET", 0, 1, tmplput_Secret, NULL, CTX_NODECONF);
+        RegisterNamespace("CFG:IGNET:HOST", 0, 1, tmplput_Host, NULL, CTX_NODECONF);
+        RegisterNamespace("CFG:IGNET:PORT", 0, 1, tmplput_Port, NULL, CTX_NODECONF);
 
 	RegisterIterator("NODECONFIG", 0, NULL, load_netconf, NULL, DeleteHash, CTX_NODECONF, CTX_NONE, IT_NOFLAG);
 }
