@@ -1,5 +1,5 @@
 /*
- * $Id: setup.c 7364 2009-04-17 18:34:37Z ajc $
+ * $Id: setup.c 8069 2009-11-20 04:03:45Z ajc $
  *
  * WebCit setup utility
  * 
@@ -23,15 +23,17 @@ char suggested_url[SIZ];
 
 /* some copies... */
 int lprintf(int loglevel, const char *format, ...){return 0;}
+void wc_printf(const char *format,...){}
+
 void RegisterNS(const char *NSName, long len, 
 		int nMinArgs, 
 		int nMaxArgs, 
 		WCHandlerFunc HandlerFunc,
+		WCPreevalFunc PreEvalFunc,
 		int ContextRequired){}
+void RegisterHeaderHandler(const char *Name, long Len, Header_Evaluator F){}
 pthread_key_t MyConKey;
 
-#include "wc_gettext.h"
-
 #ifdef ENABLE_NLS
 
 #ifdef HAVE_USELOCALE 
@@ -53,6 +55,7 @@ void delete_init_entry(char *which_entry)
 	char levels[SIZ];
 	char state[SIZ];
 	char prog[SIZ];
+	int rv;
 
 	inittab = strdup("");
 	if (inittab == NULL) return;
@@ -84,7 +87,7 @@ void delete_init_entry(char *which_entry)
 	fclose(fp);
 	fp = fopen("/etc/inittab", "w");
 	if (fp != NULL) {
-		fwrite(inittab, strlen(inittab), 1, fp);
+		rv = fwrite(inittab, strlen(inittab), 1, fp);
 		fclose(fp);
 		kill(1, SIGHUP);	/* Tell init to re-read /etc/inittab */
 	}
@@ -162,7 +165,7 @@ int yesno(char *question, int default_value)
 {
 	int i = 0;
 	int answer = 0;
-	char buf[SIZ];
+	char buf[SIZ] = "";
 
 	switch (setup_type) {
 
@@ -172,18 +175,17 @@ int yesno(char *question, int default_value)
 				question,
 				( default_value ? "Yes" : "No" )
 			);
-			if (fgets(buf, sizeof buf, stdin)) {
+			if (fgets(buf, sizeof buf, stdin))
+			{
 				answer = tolower(buf[0]);
+				if ((buf[0]==0) || (buf[0]==13) || (buf[0]==10))
+					answer = default_value;
+				else if (answer == 'y')
+					answer = 1;
+				else if (answer == 'n')
+					answer = 0;
 			}
-			else {
-				answer = default_value;
-			}
-			if ((buf[0]==0) || (buf[0]==13) || (buf[0]==10))
-				answer = default_value;
-			else if (answer == 'y')
-				answer = 1;
-			else if (answer == 'n')
-				answer = 0;
+
 		} while ((answer < 0) || (answer > 1));
 		break;
 
@@ -210,10 +212,11 @@ int yesno(char *question, int default_value)
 
 void set_value(char *prompt, char str[])
 {
-	char buf[SIZ];
+	char buf[SIZ] = "";
 	char dialog_result[PATH_MAX];
 	char setupmsg[SIZ];
 	FILE *fp;
+	int rv;
 
 	strcpy(setupmsg, "");
 
@@ -223,12 +226,12 @@ void set_value(char *prompt, char str[])
 		printf("\n%s\n", prompt);
 		printf("This is currently set to:\n%s\n", str);
 		printf("Enter new value or press return to leave unchanged:\n");
-		fgets(buf, sizeof buf, stdin);
-		buf[strlen(buf) - 1] = 0;
+		if (fgets(buf, sizeof buf, stdin)) {
+			buf[strlen(buf) - 1] = 0;
+		}
 		if (strlen(buf) != 0)
 			strcpy(str, buf);
 		break;
-
 	case UI_DIALOG:
 		CtdlMakeTempFileName(dialog_result, sizeof dialog_result);
 		sprintf(buf, "exec %s --inputbox '%s' 19 72 '%s' 2>%s",
@@ -236,12 +239,13 @@ void set_value(char *prompt, char str[])
 			prompt,
 			str,
 			dialog_result);
-		system(buf);
+		rv = system(buf);
 		fp = fopen(dialog_result, "r");
 		if (fp != NULL) {
-			fgets(str, sizeof buf, fp);
-			if (str[strlen(str)-1] == 10) {
-				str[strlen(str)-1] = 0;
+			if (fgets(str, sizeof buf, fp)){
+				if (str[strlen(str)-1] == 10) {
+					str[strlen(str)-1] = 0;
+				}
 			}
 			fclose(fp);
 			unlink(dialog_result);
@@ -252,17 +256,23 @@ void set_value(char *prompt, char str[])
 }
 
 
-
+extern const char *AvailLang[];
 int GetLocalePrefs(void)
 {
+	int nLocales;
 	StrBuf *Buf;
 	char buf[SIZ];
 	char dialog_result[PATH_MAX];
 	FILE *fp;
 	int i = 0;
 	int offs = 0;
+	int rv;
 
 
+	nLocales = 0; 
+	while (!IsEmptyStr(AvailLang[nLocales]))
+		nLocales++;
+
 	Buf = NewStrBuf();
 
 	StrBufAppendBufPlain(Buf, HKEY("Select the locale webcit should use : \n"), 0);
@@ -270,7 +280,7 @@ int GetLocalePrefs(void)
 	StrBufAppendBufPlain(Buf, HKEY(" 0 Let the user select it at the login prompt (default)\n"), 0);
 	offs ++;
 #endif
-	for (i = 0; i < NUM_LANGS; i++) {
+	for (i = 0; i < nLocales; i++) {
 		StrBufAppendPrintf(Buf, " %ld: %s\n", i + offs, AvailLang[i]);
 
 	}
@@ -281,8 +291,8 @@ int GetLocalePrefs(void)
 		printf("\n%s\n", ChrPtr(Buf));
 		printf("This is currently set to:\n%ld\n", 0L);
 		printf("Enter new value or press return to leave unchanged:\n");
-		fgets(buf, sizeof buf, stdin);
-		return atoi(buf);
+		if (fgets(buf, sizeof buf, stdin))
+			return atoi(buf);
 		break;
 
 	case UI_DIALOG:
@@ -292,13 +302,14 @@ int GetLocalePrefs(void)
 			ChrPtr(Buf),
 			0L,
 			dialog_result);
-		system(buf);
+		rv = system(buf);
 		fp = fopen(dialog_result, "r");
 		if (fp != NULL) {
 			char *str = &buf[0];
-			fgets(str, sizeof buf, fp);
-			if (str[strlen(str)-1] == 10) {
-				str[strlen(str)-1] = 0;
+			if (fgets(str, sizeof buf, fp)){
+				if (str[strlen(str)-1] == 10) {
+					str[strlen(str)-1] = 0;
+				}
 			}
 			fclose(fp);
 			unlink(dialog_result);
@@ -313,6 +324,7 @@ int GetLocalePrefs(void)
 void important_message(char *title, char *msgtext)
 {
 	char buf[SIZ];
+	int rv;
 
 	switch (setup_type) {
 
@@ -320,14 +332,14 @@ void important_message(char *title, char *msgtext)
 		printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
 		printf("       %s \n\n%s\n\n", title, msgtext);
 		printf("Press return to continue...");
-		fgets(buf, sizeof buf, stdin);
+		if (fgets(buf, sizeof buf, stdin));
 		break;
 
 	case UI_DIALOG:
 		sprintf(buf, "exec %s --msgbox '%s' 19 72",
 			getenv("CTDL_DIALOG"),
 			msgtext);
-		system(buf);
+		rv = system(buf);
 		break;
 	}
 }
@@ -423,11 +435,12 @@ void install_init_scripts(void)
 	struct stat etcinitd;
 	FILE *fp;
 	char *initfile = "/etc/init.d/webcit";
+	int rv;
 
 	fp = fopen(initfile, "r");
 	if (fp != NULL) {
 		if (yesno("WebCit already appears to be configured to start at boot.\n"
-		   "Would you like to keep your boot configuration as is?\n", 1) == 1) {
+			  "Would you like to keep your boot configuration as is?\n", 1) == 1) {
 			return;
 		}
 		fclose(fp);
@@ -436,8 +449,8 @@ void install_init_scripts(void)
 
 	/* Otherwise, prompt the user to create an entry. */
 	snprintf(question, sizeof question,
-		"Would you like to automatically start WebCit at boot?"
-	);
+		 "Would you like to automatically start WebCit at boot?"
+		);
 	if (yesno(question, 1) == 0)
 		return;
 
@@ -495,20 +508,20 @@ void install_init_scripts(void)
 
 	/* Now ask for the port numbers */
 	snprintf(question, sizeof question,
-		"On which port do you want WebCit to listen for HTTP "
-		"requests?\n\nYou can use the standard port (80) if you are "
-		"not running another\nweb server (such as Apache), otherwise "
-		"select another port.");
+		 "On which port do you want WebCit to listen for HTTP "
+		 "requests?\n\nYou can use the standard port (80) if you are "
+		 "not running another\nweb server (such as Apache), otherwise "
+		 "select another port.");
 	set_value(question, http_port);
 	uname(&my_utsname);
 	sprintf(suggested_url, "http://%s:%s/", my_utsname.nodename, http_port);
 
 #ifdef HAVE_OPENSSL
 	snprintf(question, sizeof question,
-		"On which port do you want WebCit to listen for HTTPS "
-		"requests?\n\nYou can use the standard port (443) if you are "
-		"not running another\nweb server (such as Apache), otherwise "
-		"select another port.");
+		 "On which port do you want WebCit to listen for HTTPS "
+		 "requests?\n\nYou can use the standard port (443) if you are "
+		 "not running another\nweb server (such as Apache), otherwise "
+		 "select another port.");
 	set_value(question, https_port);
 #endif
 
@@ -519,7 +532,7 @@ void install_init_scripts(void)
 	}
 	else {
 		snprintf(question, sizeof question,
-			"Is the Citadel service running on the same host as WebCit?");
+			 "Is the Citadel service running on the same host as WebCit?");
 		if (yesno(question, ((!strcasecmp(hostname, "uds")) ? 1 : 0))) {
 			strcpy(hostname, "uds");
 			if (atoi(portname) != 0) strcpy(portname, "/usr/local/citadel");
@@ -529,9 +542,9 @@ void install_init_scripts(void)
 			if (!strcasecmp(hostname, "uds")) strcpy(hostname, "127.0.0.1");
 			if (atoi(portname) == 0) strcpy(portname, "504");
 			set_value("Enter the host name or IP address of your "
-				"Citadel server.", hostname);
+				  "Citadel server.", hostname);
 			set_value("Enter the port number on which Citadel is "
-				"running (usually 504)", portname);
+				  "running (usually 504)", portname);
 		}
 	}
 
@@ -539,7 +552,10 @@ void install_init_scripts(void)
 	fp = fopen(initfile, "w");
 
 	fprintf(fp,	"#!/bin/sh\n"
-			"\n"
+		"\n"
+		"# uncomment this to create coredumps as described in\n"
+		"# http://www.citadel.org/doku.php/faq:mastering_your_os:gdb#how.do.i.make.my.system.produce.core-files\n"
+		"# ulimit -c unlimited\n"
 			"WEBCIT_DIR=%s\n", setup_directory);
 	fprintf(fp,	"HTTP_PORT=%s\n", http_port);
 #ifdef HAVE_OPENSSL
@@ -621,11 +637,11 @@ void install_init_scripts(void)
 	chmod(initfile, 0755);
 
 	/* Set up the run levels. */
-	system("/bin/rm -f /etc/rc?.d/[SK]??webcit 2>/dev/null");
+	rv = system("/bin/rm -f /etc/rc?.d/[SK]??webcit 2>/dev/null");
 	snprintf(command, sizeof(command), "for x in 2 3 4 5 ; do [ -d /etc/rc$x.d ] && ln -s %s /etc/rc$x.d/S84webcit ; done 2>/dev/null", initfile);
-	system(command);
+	rv = system(command);
 	snprintf(command, sizeof(command), "for x in 0 6 S; do [ -d /etc/rc$x.d ] && ln -s %s /etc/rc$x.d/K15webcit ; done 2>/dev/null", initfile);
-	system(command);
+	rv = system(command);
 
 }
 
@@ -656,6 +672,7 @@ int main(int argc, char *argv[])
 	char aaa[256];
 	int info_only = 0;
 	strcpy(suggested_url, "http://<your_host_name>:<port>/");
+	int rv;
 
 	/* set an invalid setup type */
 	setup_type = (-1);
@@ -732,8 +749,8 @@ int main(int argc, char *argv[])
 		install_init_scripts();
 
 		if (!access("/etc/init.d/webcit", X_OK)) {
-			system("/etc/init.d/webcit stop");
-			system("/etc/init.d/webcit start");
+			rv = system("/etc/init.d/webcit stop");
+			rv = system("/etc/init.d/webcit start");
 		}
 
 		sprintf(aaa,
