@@ -1,5 +1,5 @@
 /* 
- * $Id: smtpqueue.c 7369 2009-04-18 12:27:01Z dothebart $
+ * $Id: smtpqueue.c 7939 2009-10-23 12:51:53Z dothebart $
  *
  * Display the outbound SMTP queue
  */
@@ -129,47 +129,49 @@ void display_queue_msg(long msgnum)
 
 	}
 
-	wprintf("<tr><td>");
-	wprintf("%ld<br />", msgnum);
-	wprintf(" <a href=\"javascript:DeleteQueueMsg(%ld,%ld);\">%s</a>", 
+	wc_printf("<tr><td>");
+	wc_printf("%ld<br />", msgnum);
+	wc_printf(" <a href=\"javascript:DeleteSMTPqueueMsg(%ld,%ld);\">%s</a>", 
 		msgnum, msgid, _("(Delete)")
 	);
 
-	wprintf("</td><td>");
+	wc_printf("</td><td>");
 	if (submitted > 0) {
 		webcit_fmt_date(buf, 1024, submitted, 1);
-		wprintf("%s", buf);
+		wc_printf("%s", buf);
 	}
 	else {
-		wprintf("&nbsp;");
+		wc_printf("&nbsp;");
 	}
 
-	wprintf("</td><td>");
+	wc_printf("</td><td>");
 	if (last_attempt > 0) {
 		webcit_fmt_date(buf, 1024, last_attempt, 1);
-		wprintf("%s", buf);
+		wc_printf("%s", buf);
 	}
 	else {
-		wprintf("&nbsp;");
+		wc_printf("&nbsp;");
 	}
 
-	wprintf("</td><td>");
+	wc_printf("</td><td>");
 	escputs(sender);
 
-	wprintf("</td><td>");
-	wprintf("%s", recipients);
-	wprintf("</td></tr>\n");
+	wc_printf("</td><td>");
+	wc_printf("%s", recipients);
+	wc_printf("</td></tr>\n");
 
 }
 
 
 void display_smtpqueue_inner_div(void) {
-	message_summary *Msg;
+	message_summary *Msg = NULL;
 	wcsession *WCC = WC;
 	int i;
 	int num_msgs;
 	StrBuf *Buf;
+	SharedMessageStatus Stat;
 
+	memset(&Stat, 0, sizeof(SharedMessageStatus));
 	/* Check to see if we can go to the __CitadelSMTPspoolout__ room.
 	 * If not, we don't have access to the queue.
 	 */
@@ -178,96 +180,82 @@ void display_smtpqueue_inner_div(void) {
 	FreeStrBuf(&Buf);
 	if (!strcasecmp(ChrPtr(WCC->wc_roomname), "__CitadelSMTPspoolout__")) {
 
-		num_msgs = load_msg_ptrs("MSGS ALL", 0);
+		Stat.maxload = 10000;
+		Stat.lowest_found = (-1);
+		Stat.highest_found = (-1);
+		num_msgs = load_msg_ptrs("MSGS ALL", &Stat);
 		if (num_msgs > 0) {
-                        wprintf("<table class=\"mailbox_summary\" rules=rows "
+                        wc_printf("<table class=\"mailbox_summary\" rules=rows "
                         	"cellpadding=2 style=\"width:100%%;\">"
 			);
 
-			wprintf("<tr><td><b><i>");
-			wprintf(_("Message ID"));
-			wprintf("</i></b></td><td><b><i>");
-			wprintf(_("Date/time submitted"));
-			wprintf("</i></b></td><td><b><i>");
-			wprintf(_("Last attempt"));
-			wprintf("</i></b></td><td><b><i>");
-			wprintf(_("Sender"));
-			wprintf("</i></b></td><td><b><i>");
-			wprintf(_("Recipients"));
-			wprintf("</i></b></td></tr>\n");
+			wc_printf("<tr><td><b><i>");
+			wc_printf(_("Message ID"));
+			wc_printf("</i></b></td><td><b><i>");
+			wc_printf(_("Date/time submitted"));
+			wc_printf("</i></b></td><td><b><i>");
+			wc_printf(_("Last attempt"));
+			wc_printf("</i></b></td><td><b><i>");
+			wc_printf(_("Sender"));
+			wc_printf("</i></b></td><td><b><i>");
+			wc_printf(_("Recipients"));
+			wc_printf("</i></b></td></tr>\n");
 
 			for (i=0; i<num_msgs; ++i) {
 				Msg = GetMessagePtrAt(i, WCC->summ);
-
-				display_queue_msg((Msg==NULL)? 0 : Msg->msgnum);
+				if (Msg != NULL) {
+					display_queue_msg(Msg->msgnum);
+				}
 			}
 
-			wprintf("</table>");
+			wc_printf("</table>");
 
 		}
 		else {
-			wprintf("<br /><br /><div align=\"center\">");
-			wprintf(_("The queue is empty."));
-			wprintf("</div><br /><br />");
+			wc_printf("<br /><br /><div align=\"center\">");
+			wc_printf(_("The queue is empty."));
+			wc_printf("</div><br /><br />");
 		}
 	}
 	else {
-		wprintf("<br /><br /><div align=\"center\">");
-		wprintf(_("You do not have permission to view this resource."));
-		wprintf("</div><br /><br />");
+		wc_printf("<br /><br /><div align=\"center\">");
+		wc_printf(_("You do not have permission to view this resource."));
+		wc_printf("</div><br /><br />");
 	}
-
+	output_headers(0, 0, 0, 0, 0, 0);
+	end_burst();
 }
 
-/**
- * \brief display the outbound SMTP queue
+/*
+ * display the outbound SMTP queue
  */
 void display_smtpqueue(void)
 {
 	output_headers(1, 1, 2, 0, 0, 0);
 
-	wprintf("<script type=\"text/javascript\">				\n"
-		"function RefreshQueueDisplay() {				\n"
-		"	new Ajax.Updater('smtpqueue_inner_div', 		\n"
-		"	'display_smtpqueue_inner_div', { method: 'get',		\n"
-		"		parameters: Math.random() } );			\n"
-		"}								\n"
-		"								\n"
-		"function DeleteQueueMsg(msgnum1, msgnum2) {					\n"
- 		"	new Ajax.Request(							\n"
-		"		'ajax_servcmd', {						\n"
-		"			method: 'post',						\n"
-		"			parameters: 'g_cmd=DELE ' + msgnum1 + ',' + msgnum2,	\n"
-		"			onComplete: RefreshQueueDisplay()			\n"
-		"		}								\n"
-		"	);									\n"
-		"}										\n"
-		"								\n"
-		"</script>							\n"
-	);
+	wc_printf("<div id=\"banner\">\n");
+	wc_printf("<h1>");
+	wc_printf(_("View the outbound SMTP queue"));
+	wc_printf("</h1>\n");
+	wc_printf("</div>\n");
 
-	wprintf("<div id=\"banner\">\n");
-	wprintf("<h1>");
-	wprintf(_("View the outbound SMTP queue"));
-	wprintf("</h1>\n");
-	wprintf("</div>\n");
+	wc_printf("<div id=\"content\" class=\"service\">\n");
 
-	wprintf("<div id=\"content\" class=\"service\">\n");
-
-	wprintf("<div class=\"fix_scrollbar_bug\">"
+	wc_printf("<div class=\"fix_scrollbar_bug\">"
 		"<table class=\"smtpqueue_background\">"
 		"<tr><td valign=top>\n");
 
-	wprintf("<div id=\"smtpqueue_inner_div\">");
-
-	display_smtpqueue_inner_div();
-
-	wprintf("</div>"
+	wc_printf("<div id=\"smtpqueue_inner_div\">"
+		"<div align=\"center\"><img src=\"static/throbber.gif\"></div>"
+		"</div>"
 		"<div align=\"center\">"
-		"<a href=\"javascript:RefreshQueueDisplay();\">%s</a>"
+		"<a href=\"javascript:RefreshSMTPqueueDisplay();\">%s</a>"
 		"</div>"
 		"</td></tr></table></div>\n", _("Refresh this page")
 	);
+
+	StrBufAppendPrintf(WC->trailing_javascript, "RefreshSMTPqueueDisplay();\n");
+
 	wDumpContent(1);
 
 }
@@ -276,8 +264,6 @@ void
 InitModule_SMTP_QUEUE
 (void)
 {
-	WebcitAddUrlHandler(HKEY("display_smtpqueue"), display_smtpqueue, 0);
-	WebcitAddUrlHandler(HKEY("display_smtpqueue_inner_div"), display_smtpqueue_inner_div, 0);
+	WebcitAddUrlHandler(HKEY("display_smtpqueue"), "", 0, display_smtpqueue, 0);
+	WebcitAddUrlHandler(HKEY("display_smtpqueue_inner_div"), "", 0, display_smtpqueue_inner_div, 0);
 }
-
-/*@}*/
