@@ -1,5 +1,5 @@
 
-/* $Id: webcit.h 7528 2009-06-02 20:47:33Z ajc $ */
+/* $Id: webcit.h 8127 2009-12-16 21:27:59Z dothebart $ */
 
 #include "sysdep.h"
 
@@ -87,11 +87,10 @@
 #include "sysdep.h"
 
 #include "subst.h"
-#include "wc_gettext.h"
 #include "messages.h"
 #include "paramhandling.h"
 #include "preferences.h"
-
+#include "roomops.h"
 
 #ifdef HAVE_OPENSSL
 /* Work around RedHat's b0rken OpenSSL includes */
@@ -99,8 +98,11 @@
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
+extern char *ssl_cipher_list;
+#define	DEFAULT_SSL_CIPHER_LIST	"DEFAULT"	/* See http://openssl.org/docs/apps/ciphers.html */
 #endif
 
+
 #define CALENDAR_ROOM_NAME	"Calendar"
 #define PRODID "-//Citadel//NONSGML Citadel Calendar//EN"
 
@@ -113,9 +115,9 @@
 #define PORT_NUM		2000		/* port number to listen on */
 #define DEVELOPER_ID		0
 #define CLIENT_ID		4
-#define CLIENT_VERSION		751		/* This version of WebCit */
-#define MINIMUM_CIT_VERSION	744		/* min required Citadel ver */
-#define	LIBCITADEL_MIN		744		/* min required libcitadel ver */
+#define CLIENT_VERSION		770		/* This version of WebCit */
+#define MINIMUM_CIT_VERSION	770		/* min required Citadel ver */
+#define	LIBCITADEL_MIN		766		/* min required libcitadel ver */
 #define DEFAULT_HOST		"localhost"	/* Default Citadel server */
 #define DEFAULT_PORT		"504"
 #define TARGET			"webcit01"	/* Target for inline URL's */
@@ -173,6 +175,9 @@
 #define UA_GOTOALLOWED	4
 #define UA_HASNEWMSGS	8
 #define UA_ZAPPED	16
+#define UA_POSTALLOWED          32
+#define UA_ADMINALLOWED         64
+#define UA_DELETEALLOWED        128
 
 
 /*
@@ -195,6 +200,42 @@
 			US_NOPROMPT | US_DISAPPEAR | US_PAGINATOR | \
 			US_FLOORS | US_COLOR | US_PROMPTCTL )
 
+
+#define MAJORCODE(a) (((int)(a / 100) ) * 100)
+
+#define LISTING_FOLLOWS	100
+#define CIT_OK 		200	
+#define MORE_DATA 	300
+#define SEND_LISTING 	400
+#define ERROR 		500
+#define BINARY_FOLLOWS 	600
+#define SEND_BINARY 	700
+#define START_CHAT_MODE	800
+#define ASYNC_MSG 	900
+
+#define MINORCODE(a) (a % 100)
+#define ASYNC_GEXP 			02	
+#define INTERNAL_ERROR 			10	
+#define TOO_BIG 			11	
+#define ILLEGAL_VALUE 			12	
+#define NOT_LOGGED_IN 			20	
+#define CMD_NOT_SUPPORTED 		30	
+#define SERVER_SHUTTING_DOWN 		31	
+#define PASSWORD_REQUIRED 		40	
+#define ALREADY_LOGGED_IN 		41	
+#define USERNAME_REQUIRED 		42	
+#define HIGHER_ACCESS_REQUIRED 		50	
+#define MAX_SESSIONS_EXCEEDED 		51	
+#define RESOURCE_BUSY 			52	
+#define RESOURCE_NOT_OPEN 		53	
+#define NOT_HERE 			60	
+#define INVALID_FLOOR_OPERATION 	61	
+#define NO_SUCH_USER 			70	
+#define FILE_NOT_FOUND 			71	
+#define ROOM_NOT_FOUND 			72	
+#define NO_SUCH_SYSTEM 			73	
+#define ALREADY_EXISTS 			74	
+#define MESSAGE_NOT_FOUND 		75
 /*
  * NLI is the string that shows up in a who's online listing for sessions
  * that are active, but for which no user has yet authenticated.
@@ -203,16 +244,16 @@
 
 
 /*
- * \brief	Linked list of session variables encoded in an x-www-urlencoded content type
+ * Linked list of session variables encoded in an x-www-urlencoded content type
  */
 typedef struct urlcontent urlcontent;
 struct urlcontent {
-	char url_key[32];          /* the variable name */
-	StrBuf *url_data;            /* its value */
+	char url_key[32];		/* key */
+	StrBuf *url_data;		/* value */
 };
 
 /*
- * \brief information about us ???
+ * Information about the Citadel server to which we are connected
  */ 
 typedef struct _serv_info {
 	int serv_pid;			/* Process ID of the Citadel server */
@@ -224,7 +265,6 @@ typedef struct _serv_info {
 	StrBuf *serv_bbs_city;		/* Geographic location of the Citadel server */
 	StrBuf *serv_sysadm;		/* Name of system administrator */
 	StrBuf *serv_moreprompt;	/* Whats the commandline textprompt */
-	int serv_ok_floors;		/* nonzero == server supports floors */
 	int serv_supports_ldap;		/* is the server linked against an ldap tree for adresses? */
 	int serv_newuser_disabled;	/* Has the server disabled self-service new user creation? */
 	StrBuf *serv_default_cal_zone;  /* Default timezone for unspecified calendar items */
@@ -235,50 +275,6 @@ typedef struct _serv_info {
 } ServInfo;
 
 
-
-/*
- * \brief This struct holds a list of rooms for \\\<G\\\>oto operations.
- */
-struct march {
-	struct march *next;       /* pointer to next in linked list */
-	char march_name[128];     /* name of room */
-	int march_floor;          /* floor number of room */
-	int march_order;          /* sequence in which we are to visit this room */
-};
-
-/* *
- * \brief	This struct holds a list of rooms for client display.
- *		It is a binary tree.
- */
-struct roomlisting {
-	struct roomlisting *lnext;	/* pointer to 'left' tree node */
-	struct roomlisting *rnext;	/* pointer to 'right' tree node */
-	char rlname[128];		/* name of room */
-	unsigned rlflags;		/* room flags */
-	int rlfloor;			/* the floor it resides on */
-	int rlorder;			/* room listing order */
-};
-
-
-
-
-
-
-
-/**
- * \brief  Data structure for roomlist-to-folderlist conversion 
- */
-struct folder {
-	int floor;      /* which floor is it on */
-	char room[SIZ];	/* which roomname ??? */
-	char name[SIZ];	/* which is its own name??? */
-	int hasnewmsgs;	/* are there unread messages inside */
-	int is_mailbox;	/* is it a mailbox?  */
-	int selectable;	/* can we select it ??? */
-	int view;       /* whats its default view? inbox/calendar.... */
-	int num_rooms;	/* If this is a floor, how many rooms does it have */
-};
-
 typedef struct _disp_cal {					
 	icalcomponent *cal;		/* cal items for display */
 	long cal_msgnum;		/* cal msgids for display */
@@ -290,20 +286,49 @@ typedef struct _disp_cal {
 
 	int multi_day_event;
 	int is_repeat;
+	icalcomponent *SortBy;		/* cal items for display */
+	icalproperty_status Status;
 } disp_cal;						
 
-
+typedef struct _IcalEnumMap {
+	const char *Name;
+	long NameLen;
+	icalproperty_kind map;
+} IcalEnumMap;
 
 /*
  * Address book entry (keep it short and sweet, it's just a quickie lookup
  * which we can use to get to the real meat and bones later)
  */
 typedef struct _addrbookent {
-	char ab_name[64]; /**< name string */
-	long ab_msgnum;   /**< message number of address book entry */
+	char ab_name[64];	/* name string */
+	long ab_msgnum;		/* message number of address book entry */
 } addrbookent;
 
 
+#define AJAX (1<<0)
+#define ANONYMOUS (1<<1)
+#define NEED_URL (1<<2)
+#define XHTTP_COMMANDS (1<<3)
+#define BOGUS (1<<4)
+#define URLNAMESPACE (1<<4)
+#define LOGCHATTY (1<<5)
+#define COOKIEUNNEEDED (1<<6)
+#define ISSTATIC (1<<7)
+#define FORCE_SESSIONCLOSE (1<<8)
+#define PARSE_REST_URL (1<<0)
+
+typedef void (*WebcitHandlerFunc)(void);
+typedef struct  _WebcitHandler{
+	WebcitHandlerFunc F;
+	long Flags;
+	StrBuf *Name;
+	StrBuf *DisplayName;
+} WebcitHandler;
+
+
+void WebcitAddUrlHandler(const char * UrlString, long UrlSLen, const char *DisplayName, long dslen, WebcitHandlerFunc F, long Flags);
+
 typedef struct _headereval {
 	ExamineMsgHeaderFunc evaluator;
 	int Type;
@@ -322,6 +347,74 @@ enum {
 	eNone
 };
 
+enum {
+	eGET,
+	ePOST,
+	eOPTIONS,
+	ePROPFIND,
+	ePUT,
+	eDELETE,
+	eHEAD,
+	eMOVE,
+	eCOPY,
+	eNONE
+};
+const char *ReqStrs[eNONE];
+
+#define NO_AUTH 0
+#define AUTH_COOKIE 1
+#define AUTH_BASIC 2
+
+
+
+typedef struct _HdrRefs {
+	long eReqType;				/* HTTP method */
+	int desired_session;
+	int SessionKey;
+
+	int got_auth;
+	int DontNeedAuth;
+	long ContentLength;
+	time_t if_modified_since;
+	int gzip_ok;				/* Nonzero if Accept-encoding: gzip */
+	int prohibit_caching;
+	int dav_depth;
+	int Static;
+
+	/* these are references into Hdr->HTTPHeaders, so we don't need to free them. */
+	StrBuf *ContentType;
+	StrBuf *RawCookie;
+	StrBuf *ReqLine;
+	StrBuf *http_host;			/* HTTP Host: header */
+	StrBuf *browser_host;
+	StrBuf *browser_language;
+	StrBuf *user_agent;
+	StrBuf *plainauth;
+	StrBuf *dav_ifmatch;
+
+	const WebcitHandler *Handler;
+} HdrRefs;
+
+typedef struct _ParsedHttpHdrs {
+	int http_sock;				/* HTTP server socket */
+	const char *Pos;
+	StrBuf *ReadBuf;
+
+	StrBuf *c_username;
+	StrBuf *c_password;
+	StrBuf *c_roomname;
+	StrBuf *c_language;
+	StrBuf *this_page;			/* URL of current page */
+	StrBuf *PlainArgs; 
+
+	HashList *urlstrings;		        /* variables passed to webcit in a URL */
+	HashList *HTTPHeaders;                  /* the headers the client sent us */
+	int nWildfireHeaders;                   /* how many wildfire headers did we already send? */
+
+	HdrRefs HR;
+} ParsedHttpHdrs;
+
+
 /*
  * One of these is kept for each active Citadel session.
  * HTTP transactions are bound to one at a time.
@@ -329,122 +422,127 @@ enum {
 typedef struct wcsession wcsession;
 struct wcsession {
 /* infrastructural members */
-	wcsession *next;			/**< Linked list */
-	pthread_mutex_t SessionMutex;		/**< mutex for exclusive access */
-	int wc_session;				/**< WebCit session ID */
-	int killthis;				/**< Nonzero == purge this session */
-	int is_mobile;			        /**< Client is a handheld browser */
-	int ctdl_pid;				/**< Session ID on the Citadel server */
-	int nonce;				/**< session nonce (to prevent session riding) */
+	wcsession *next;			/* Linked list */
+	pthread_mutex_t SessionMutex;		/* mutex for exclusive access */
+	int wc_session;				/* WebCit session ID */
+	int killthis;				/* Nonzero == purge this session */
+	int is_mobile;			        /* Client is a handheld browser */
+	int ctdl_pid;				/* Session ID on the Citadel server */
+	int nonce;				/* session nonce (to prevent session riding) */
+	int SessionKey;
 
 /* Session local Members */
-	int http_sock;				/**< HTTP server socket */
-	int serv_sock;				/**< Client socket to Citadel server */
-	StrBuf *ReadBuf;
-	int chat_sock;				/**< Client socket to Citadel server - for chat */
-	time_t lastreq;				/**< Timestamp of most recent HTTP */
-	time_t last_pager_check;		/**< last time we polled for instant msgs */
-	ServInfo *serv_info;                   /**< Iformation about the citserver we're connected to */
-/* Request local Members */
-	StrBuf *CLineBuf;                       /**< linebuffering client stuff */
-	StrBuf *UrlFragment1;                   /**< first urlfragment, if NEED_URL is specified by the handler*/
-	StrBuf *UrlFragment2;                   /**< second urlfragment, if NEED_URL is specified by the handler*/
-	StrBuf *UrlFragment3;                   /**< third urlfragment, if NEED_URL is specified by the handler*/
-	StrBuf *UrlFragment4;                   /**< fourth urlfragment, if NEED_URL is specified by the handler*/
-	StrBuf *WBuf;                           /**< Our output buffer */
-	StrBuf *HBuf;                           /**< Our HeaderBuffer */
-	StrBuf *this_page;			/**< URL of current page */
-	HashList *urlstrings;		        /**< variables passed to webcit in a URL */
-	HashList *vars; 			/**< HTTP variable substitutions for this page */
-	StrBuf *http_host;			/**< HTTP Host: header */
-	int is_ajax;                            /** < are we doing an ajax request? */
-	int gzip_ok;				/**< Nonzero if Accept-encoding: gzip */
-	int nWildfireHeaders;                   /**< how many wildfire headers did we already send? */
+	int serv_sock;				/* Client socket to Citadel server */
+	StrBuf *ReadBuf;                        /* here we keep our stuff while reading linebuffered from the server. */
+	StrBuf *MigrateReadLineBuf;             /* here we buffer legacy server read stuff */
+	const char *ReadPos;                    /* whats our read position in ReadBuf? */
+	int chat_sock;				/* Client socket to Citadel server - for chat */
+	time_t lastreq;				/* Timestamp of most recent HTTP */
+	time_t last_pager_check;		/* last time we polled for instant msgs */
+	ServInfo *serv_info;			/* Information about the citserver we're connected to */
+	int is_ajax;                            /* are we doing an ajax request? */
 
-	StrBuf *trailing_javascript;		/**< extra javascript to be appended to page */
-	char ImportantMessage[SIZ];		/**< ??? todo */
+/* Request local Members */
+	StrBuf *CLineBuf;                       /* linebuffering client stuff */
+	ParsedHttpHdrs *Hdr;
+	StrBuf *WBuf;                           /* Our output buffer */
+	StrBuf *HBuf;                           /* Our HeaderBuffer */
+
+	HashList *vars; 			/* HTTP variable substitutions for this page */
+	StrBuf *trailing_javascript;		/* extra javascript to be appended to page */
+	char ImportantMessage[SIZ];
+	StrBuf *ImportantMsg;
+	HashList *Directory;			/* Parts of the directory URL in snippets */
+	const floor *CurrentFloor;              /**< when Parsing REST, which floor are we on? */
 
 /* accounting */
-	StrBuf *wc_username;			/**< login name of current user */
-	StrBuf *httpauth_user;  		/**< only for GroupDAV sessions */
-	StrBuf *wc_fullname;			/**< Screen name of current user */
-	StrBuf *wc_password;			/**< Password of current user */
-	StrBuf *httpauth_pass;  		/**< only for GroupDAV sessions */
-	int axlevel;				/**< this user's access level */
-	int is_aide;				/**< nonzero == this user is an Aide */
-	int is_room_aide;			/**< nonzero == this user is a Room Aide in this room */
-	int connected;				/**< nonzero == we are connected to Citadel */
-	int logged_in;				/**< nonzero == we are logged in  */
-	int need_regi;				/**< This user needs to register. */
-	int need_vali;				/**< New users require validation. */
+	StrBuf *wc_username;			/* login name of current user */
+	StrBuf *wc_fullname;			/* Screen name of current user */
+	StrBuf *wc_password;			/* Password of current user */
+	StrBuf *httpauth_pass;  		/* only for GroupDAV sessions */
+	int axlevel;				/* this user's access level */
+	int is_aide;				/* nonzero == this user is an Aide */
+	int is_room_aide;			/* nonzero == this user is a Room Aide in this room */
+	int connected;				/* nonzero == we are connected to Citadel */
+	int logged_in;				/* nonzero == we are logged in  */
+	int need_regi;				/* This user needs to register. */
+	int need_vali;				/* New users require validation. */
 
 /* Preferences */
-	char cs_inet_email[256];		/**< User's preferred Internet addr. */
-	char reply_to[512];			/**< reply-to address */
-	HashList *hash_prefs;			/**< WebCit preferences for this user */
-	StrBuf *DefaultCharset;                 /**< Charset the user preferes */
-	int downloaded_prefs;                   /** Has the client download its prefs yet? */
-	int SavePrefsToServer;                  /**< Should we save our preferences to the server at the end of the request? */
-	int selected_language;			/**< Language selected by user */
-	int time_format_cache;                  /**< which timeformat does our user like? */
+	StrBuf *cs_inet_email;  		/* User's preferred Internet addr. */
+	char reply_to[512];			/* reply-to address */
+	HashList *hash_prefs;			/* WebCit preferences for this user */
+	StrBuf *DefaultCharset;                 /* Charset the user preferes */
+	int downloaded_prefs;                   /* Has the client download its prefs yet? */
+	int SavePrefsToServer;                  /* Should we save our preferences to the server at the end of the request? */
+	int selected_language;			/* Language selected by user */
+	int time_format_cache;                  /* which timeformat does our user like? */
 
 /* current room related */
-	StrBuf *wc_roomname;			/**< Room we are currently in */
-	unsigned room_flags;			/**< flags associated with the current room */
-	unsigned room_flags2;			/**< flags associated with the current room */
-	int wc_view;				/**< view for the current room */
-	int wc_default_view;			/**< default view for the current room */
-	int wc_is_trash;			/**< nonzero == current room is a Trash folder */
-	int wc_floor;				/**< floor number of current room */
-	int is_mailbox;				/**< the current room is a private mailbox */
+	StrBuf *wc_roomname;			/* Room we are currently in */
+	unsigned room_flags;			/* flags associated with the current room */
+	unsigned room_flags2;			/* flags associated with the current room */
+	int wc_view;				/* view for the current room */
+	int wc_default_view;			/* default view for the current room */
+	int wc_is_trash;			/* nonzero == current room is a Trash folder */
+	int wc_floor;				/* floor number of current room */
+	int is_mailbox;				/* the current room is a private mailbox */
 
 /* next/previous room thingabob */
-	struct march *march;			/**< march mode room list */
-	char ugname[128];			/**< where does 'ungoto' take us */
-	long uglsn;				/**< last seen message number for ungoto */
+	struct march *march;			/* march mode room list */
+	char ugname[128];			/* where does 'ungoto' take us */
+	long uglsn;				/* last seen message number for ungoto */
 
 /* Uploading; mime attachments for composing messages */
-	HashList *attachments;             	/**< list of attachments for 'enter message' */
-	int upload_length;			/**< content length of http-uploaded data */
-	char *upload;				/**< pointer to http-uploaded data */
-	char upload_filename[PATH_MAX];		/**< filename of http-uploaded data */
-	char upload_content_type[256];		/**< content type of http-uploaded data */
+	HashList *attachments;             	/* list of attachments for 'enter message' */
+	int upload_length;			/* content length of http-uploaded data */
+	StrBuf *upload;				/* pointer to http-uploaded data */
+	char upload_filename[PATH_MAX];		/* filename of http-uploaded data */
+	char upload_content_type[256];		/* content type of http-uploaded data */
 
-	int new_mail;				/**< user has new mail waiting */
-	int remember_new_mail;			/**< last count of new mail messages */
+	int new_mail;				/* user has new mail waiting */
+	int remember_new_mail;			/* last count of new mail messages */
 
 /* Roomiew control */
-	HashList *summ;                         /**< list of messages for mailbox summary view */
+	HashList *Floors;                       /* floors our citserver has... */
+	HashList *summ;                         /* list of messages for mailbox summary view */
   /** Perhaps these should be within a struct instead */
-	long startmsg;                          /**< message number to start at */
-	long maxmsgs;                           /**< maximum messages to display */
-        long num_displayed;                     /**< number of messages actually displayed */
-	HashList *disp_cal_items;               /**< sorted list of calendar items; startdate is the sort criteria. */
+	long startmsg;                          /* message number to start at */
+	long maxmsgs;                           /* maximum messages to display */
+        long num_displayed;                     /* number of messages actually displayed */
+	HashList *disp_cal_items;               /* sorted list of calendar items; startdate is the sort criteria. */
 
 
-	char last_chat_user[256];		/**< ??? todo */
+	char last_chat_user[256];
 
 /* Iconbar controls */
-	struct folder *cache_fold;		/**< cache the iconbar room list */
-	int cache_max_folders;			/**< ??? todo */
-	int cache_num_floors;			/**< ??? todo */
-	time_t cache_timestamp;			/**< ??? todo */
-	HashList *IconBarSettings;             /**< which icons should be shown / not shown? */
-	const StrBuf *floordiv_expanded;	/**< which floordiv currently expanded */
+	struct __ofolder *cache_fold;		/* cache the iconbar room list */
+	int cache_max_folders;
+	int cache_num_floors;
+	time_t cache_timestamp;
+	long *IBSettingsVec;                    /* which icons should be shown / not shown? */
+	const StrBuf *floordiv_expanded;	/* which floordiv currently expanded */
 
 
 
 /* cache stuff for templates. TODO: find a smartrer way */
-	HashList *ServCfg;                      /**< cache our server config for editing */
-	HashList *InetCfg;                      /**< Our inet server config for editing */
+	HashList *ServCfg;                      /* cache our server config for editing */
+	HashList *InetCfg;                      /* Our inet server config for editing */
 
 };
 
-/* values for WC->current_iconbar */
-enum {
-	current_iconbar_menu,     /* view the icon menue */
-	current_iconbar_roomlist  /* view the roomtree */
-};
+
+typedef void (*Header_Evaluator)(StrBuf *Line, ParsedHttpHdrs *hdr);
+
+typedef struct _HttpHeader {
+	Header_Evaluator H;
+	StrBuf *Val;
+	int HaveEvaluator;
+} OneHttpHeader;
+
+void RegisterHeaderHandler(const char *Name, long Len, Header_Evaluator F);
+
+
 enum {
 	S_SELECT,
 	S_SHUTDOWN,
@@ -452,27 +550,6 @@ enum {
 };
 
 
-/*
- * calview contains data passed back and forth between the message fetching loop
- * and the calendar view renderer.
- */
-enum {
-	calview_month,
-	calview_day,
-	calview_week,
-	calview_brief,
-	calview_summary
-};
-
-struct calview {
-	int view;
-	int year;
-	int month;
-	int day;
-	time_t lower_bound;
-	time_t upper_bound;
-};
-
 #ifndef num_parms
 #define num_parms(source)		num_tokens(source, '|') 
 #endif
@@ -499,24 +576,27 @@ extern char *server_cookie;
 extern int is_https;
 extern int setup_wizard;
 extern char wizard_filename[];
-extern time_t if_modified_since;
 extern int follow_xff;
-extern HashList *HandlerHash;
-extern HashList *PreferenceHooks;
-extern HashList *ZoneHash;
-extern HashList *SortHash;
+extern int num_threads;
 
 void InitialiseSemaphores(void);
 void begin_critical_section(int which_one);
 void end_critical_section(int which_one);
 
-
-void stuff_to_cookie(char *cookie, size_t clen, int session,
-			StrBuf *user, StrBuf *pass, StrBuf *room);
-void cookie_to_stuff(StrBuf *cookie, int *session,
-		     StrBuf *user,
-		     StrBuf *pass,
-		     StrBuf *room);
+void stuff_to_cookie(char *cookie, size_t clen,
+		int session,
+		StrBuf *user,
+		StrBuf *pass,
+		StrBuf *room,
+		const char *language
+);
+void cookie_to_stuff(StrBuf *cookie,
+		int *session,
+		StrBuf *user,
+		StrBuf *pass,
+		StrBuf *room,
+		StrBuf *language
+);
 void locate_host(StrBuf *TBuf, int);
 void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_response);
 void openid_manual_create(void);
@@ -528,13 +608,15 @@ void display_main_menu(void);
 void display_aide_menu(void);
 void display_advanced_menu(void);
 void slrp_highest(void);
-ServInfo *get_serv_info(StrBuf *, char *);
+ServInfo *get_serv_info(StrBuf *, StrBuf *);
+void RegisterEmbeddableMimeType(const char *MimeType, long MTLen, int Priority);
+void CreateMimeStr(void);
+int GetConnected(void);
 void DeleteServInfo(ServInfo **FreeMe);
 int uds_connectsock(char *);
 int tcp_connectsock(char *, char *);
 int serv_getln(char *strbuf, int bufsize);
 int StrBuf_ServGetln(StrBuf *buf);
-int StrBuf_ServGetlnBuffered(StrBuf *buf);
 int GetServerStatus(StrBuf *Line, long* FullState);
 void serv_puts(const char *string);
 void who(void);
@@ -546,10 +628,6 @@ void FmOut(StrBuf *Target, char *align, StrBuf *Source);
 void pullquote_fmout(void);
 void wDumpContent(int);
 
-int Flathash(const char *str, long len);
-
-
-
 
 
 void UrlescPutStrBuf(const StrBuf *strbuf);
@@ -567,11 +645,10 @@ void output_headers(    int do_httpheaders,
 			int suppress_check,
 			int cache);
 void output_custom_content_header(const char *ctype);
-void wprintf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
+void wc_printf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
 void hprintf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
-void output_static(char *what);
+void output_static(const char* What);
 
-void print_menu_box(char* Title, char *Class, int nLines, ...);
 long stresc(char *target, long tSize, char *strbuf, int nbsp, int nolinebreaks);
 void escputs(const char *strbuf);
 void url(char *buf, size_t bufsize);
@@ -582,12 +659,10 @@ void msgescputs(char *strbuf);
 void msgescputs1(char *strbuf);
 void dump_vars(void);
 void embed_main_menu(void);
-void serv_read(char *buf, int bytes);
 
-void SetAccessCommand(long Oper);
 void do_addrbook_view(addrbookent *addrbook, int num_ab);
 void fetch_ab_name(message_summary *Msg, char **namebuf);
-void display_vcard(StrBuf *Target, const char *vcard_source, char alpha, int full, char **storename, long msgnum);
+void display_vcard(StrBuf *Target, StrBuf *vcard_source, char alpha, int full, char **storename, long msgnum);
 void jsonMessageList(void);
 void new_summary_view(void);
 void getseen(void);
@@ -595,12 +670,15 @@ void text_to_server(char *ptr);
 void text_to_server_qp(char *ptr);
 void confirm_delete_msg(void);
 void display_success(char *);
-void authorization_required(const char *message);
+void CheckAuthBasic(ParsedHttpHdrs *hdr);
+void GetAuthBasic(ParsedHttpHdrs *hdr);
 void server_to_text(void);
 void save_edit(char *description, char *enter_cmd, int regoto);
 void display_edit(char *description, char *check_cmd,
 		  char *read_cmd, char *save_cmd, int with_room_banner);
 long gotoroom(const StrBuf *gname);
+void remove_march(const StrBuf *aaa);
+void dotskip(void);
 void confirm_delete_room(void);
 void validate(void);
 void display_graphics_upload(char *, char *, char *);
@@ -609,12 +687,12 @@ void serv_gets(char *strbuf);
 void serv_write(const char *buf, int nbytes);
 void serv_putbuf(const StrBuf *string);
 void serv_printf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
-void load_floorlist(void);
+void load_floorlist(StrBuf *Buf);
 void shutdown_sessions(void);
 void do_housekeeping(void);
 void smart_goto(const StrBuf *);
 void worker_entry(void);
-void session_loop(HashList *HTTPHeaders, StrBuf *ReqLine, StrBuf *ReqType, StrBuf *ReadBuf);
+void session_loop(void);
 size_t wc_strftime(char *s, size_t max, const char *format, const struct tm *tm);
 void fmt_time(char *buf, size_t siz, time_t thetime);
 void httpdate(char *buf, time_t thetime);
@@ -632,11 +710,16 @@ char *memreadline(char *start, char *buf, int maxlen);
 char *memreadlinelen(char *start, char *buf, int maxlen, int *retlen);
 long extract_token(char *dest, const char *source, int parmnum, char separator, int maxlen);
 void remove_token(char *source, int parmnum, char separator);
-char *load_mimepart(long msgnum, char *partnum);
+StrBuf *load_mimepart(long msgnum, char *partnum);
 void MimeLoadData(wc_mime_attachment *Mime);
 int pattern2(char *search, char *patn);
-void do_edit_vcard(long, char *, char *, const char *);
-void select_user_to_edit(char *message, char *preselect);
+void do_edit_vcard(long msgnum, char *partnum, 
+		   message_summary *VCMsg,
+		   wc_mime_attachment *VCAtt,
+		   const char *return_to, 
+		   const char *force_room);
+
+void select_user_to_edit(const char *preselect);
 void delete_user(char *);
 void do_change_view(int);
 void folders(void);
@@ -645,18 +728,14 @@ void folders(void);
 
 void display_addressbook(long msgnum, char alpha);
 void offer_start_page(StrBuf *Target, WCTemplputParams *TP);
-void convenience_page(char *titlebarcolor, char *titlebarmsg, char *messagetext);
+void convenience_page(const char *titlebarcolor, const char *titlebarmsg, const char *messagetext);
 void output_html(const char *, int, int, StrBuf *, StrBuf *);
 void do_listsub(void);
 ssize_t write(int fd, const void *buf, size_t count);
 void cal_process_attachment(wc_mime_attachment *Mime);
-void load_calendar_item(message_summary *Msg, int unread, struct calview *c);
 void display_calendar(message_summary *Msg, int unread);
-void display_task(message_summary *Msg, int unread);
 void display_note(message_summary *Msg, int unread);
 void updatenote(void);
-void parse_calendar_view_request(struct calview *c);
-void render_calendar_view(struct calview *c);
 void do_tasks_view(void);
 int calendar_summary_view(void);
 void free_march_list(wcsession *wcf);
@@ -678,15 +757,9 @@ icaltimezone *get_default_icaltimezone(void);
 void display_icaltimetype_as_webform(struct icaltimetype *, char *, int);
 void icaltime_from_webform(struct icaltimetype *result, char *prefix);
 void icaltime_from_webform_dateonly(struct icaltimetype *result, char *prefix);
-void display_edit_individual_event(icalcomponent *supplied_vtodo, long msgnum, char *from,
-	int unread, struct calview *calv);
-void save_individual_event(icalcomponent *supplied_vtodo, long msgnum, char *from,
-	int unread, struct calview *calv);
-void ical_dezonify(icalcomponent *cal);
 void partstat_as_string(char *buf, icalproperty *attendee);
 icalcomponent *ical_encapsulate_subcomponent(icalcomponent *subcomp);
 void check_attendee_availability(icalcomponent *supplied_vevent);
-void do_freebusy(const char *req);
 int ical_ctdl_is_overlap(
                         struct icaltimetype t1start,
                         struct icaltimetype t1end,
@@ -697,17 +770,18 @@ int ical_ctdl_is_overlap(
 
 extern char *months[];
 extern char *days[];
-int read_server_binary(StrBuf *Ret, size_t total_len);
+int read_server_binary(StrBuf *Ret, size_t total_len, StrBuf *Buf);
 int StrBuf_ServGetBLOB(StrBuf *buf, long BlobSize);
+int StrBuf_ServGetBLOBBuffered(StrBuf *buf, long BlobSize);
 int read_server_text(StrBuf *Buf, long *nLines);
-int goto_config_room(void);
-long locate_user_vcard_in_this_room(void);
+int goto_config_room(StrBuf *Buf);
+long locate_user_vcard_in_this_room(message_summary **VCMsg,
+				    wc_mime_attachment **VCAtt);
 void sleeeeeeeeeep(int);
 void http_transmit_thing(const char *content_type, int is_static);
 long unescape_input(char *buf);
 void do_selected_iconbar(void);
 void spawn_another_worker_thread(void);
-void display_rss(const StrBuf *roomname, StrBuf *request_method);
 void StrEndTab(StrBuf *Target, int tabnum, int num_tabs);
 void StrBeginTab(StrBuf *Target, int tabnum, int num_tabs);
 void StrTabbedDialog(StrBuf *Target, int num_tabs, StrBuf *tabnames[]);
@@ -719,6 +793,7 @@ long guess_calhourformat(void);
 int get_time_format_cached (void);
 int xtoi(const char *in, size_t len);
 const char *get_selected_language(void);
+void display_wiki_pagelist(void);
 
 #define DATEFMT_FULL 0
 #define DATEFMT_BRIEF 1
@@ -732,14 +807,6 @@ void summary(void);
 int is_mobile_ua(char *user_agent);
 
 void embed_room_banner(char *, int);
-#define FLOOR_PARAM_LEN 3
-extern const char FLOOR_PARAM_NAMES[(FLOOR_PARAM_LEN + 1)][15];
-extern const int FLOOR_PARAM_NAMELEN[(FLOOR_PARAM_LEN + 1)];
-#define FPKEY(a) FLOOR_PARAM_NAMES[a], FLOOR_PARAM_NAMELEN[a]
-#define ROOM_PARAM_LEN 8
-extern const char ROOM_PARAM_NAMES[(ROOM_PARAM_LEN + 1)][20];
-extern const int ROOM_PARAM_NAMELEN[(ROOM_PARAM_LEN +1)];
-#define RPKEY(a) ROOM_PARAM_NAMES[a], ROOM_PARAM_NAMELEN[a]
 HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP);
 HashList *GetRoomListHash(StrBuf *Target, WCTemplputParams *TP);
 int SortRoomsByListOrder(const void *room1, const void *room2);
@@ -749,6 +816,12 @@ enum {
 	navbar_default
 };
 
+/* actual supported locales */
+void TmplGettext(StrBuf *Target, WCTemplputParams *TP);
+void offer_languages(StrBuf *Target, int nArgs, WCTemplateToken *Token, void *Context, int ContextType);
+void set_selected_language(const char *);
+void go_selected_language(void);
+void stop_selected_language(void);
 
 #ifdef HAVE_OPENSSL
 void init_ssl(void);
@@ -760,7 +833,7 @@ int client_read_sslbuffer(StrBuf *buf, int timeout);
 void client_write_ssl(const StrBuf *Buf);
 #endif
 
-void utf8ify_rfc822_string(char *buf);
+void utf8ify_rfc822_string(char **buf);
 
 void begin_burst(void);
 long end_burst(void);
@@ -770,18 +843,6 @@ extern char *hourname[];	/* Names of hours (12am, 1am, etc.) */
 void http_datestring(char *buf, size_t n, time_t xtime);
 
 
-typedef void (*WebcitHandlerFunc)(void);
-typedef struct  _WebcitHandler{
-	WebcitHandlerFunc F;
-	long Flags;
-} WebcitHandler;
-void WebcitAddUrlHandler(const char * UrlString, long UrlSLen, WebcitHandlerFunc F, long Flags);
-
-#define AJAX (1<<0)
-#define ANONYMOUS (1<<1)
-#define NEED_URL (1<<2)
-
-
 /* These should be empty, but we have them for testing */
 #define DEFAULT_HTTPAUTH_USER	""
 #define DEFAULT_HTTPAUTH_PASS	""
@@ -798,5 +859,3 @@ void WebcitAddUrlHandler(const char * UrlString, long UrlSLen, WebcitHandlerFunc
 #define WC_TIMEFORMAT_AMPM 1
 #define WC_TIMEFORMAT_24 2
 
-
-void LoadIconSettings(void);
