@@ -1,5 +1,5 @@
 /*
- * $Id: webserver.c 7336 2009-04-11 22:26:39Z dothebart $
+ * $Id: webserver.c 7847 2009-10-03 17:57:14Z dothebart $
  *
  * This contains a simple multithreaded TCP server manager.  It sits around
  * waiting on the specified port for incoming HTTP connections.  When a
@@ -28,11 +28,14 @@ int follow_xff = 0;		/* Follow X-Forwarded-For: header */
 int home_specified = 0;		/* did the user specify a homedir? */
 int time_to_die = 0;            /* Nonzero if server is shutting down */
 int DisableGzip = 0;
-extern void *context_loop(int*);
+extern void *context_loop(ParsedHttpHdrs *Hdr);
 extern void *housekeeping_loop(void);
 extern pthread_mutex_t SessionListMutex;
 extern pthread_key_t MyConKey;
 
+extern int ig_tcp_server(char *ip_addr, int port_number, int queue_len);
+extern int ig_uds_server(char *sockpath, int queue_len);
+
 
 char ctdl_key_dir[PATH_MAX]=SSL_DIR;
 char file_crpt_file_key[PATH_MAX]="";
@@ -40,10 +43,10 @@ char file_crpt_file_csr[PATH_MAX]="";
 char file_crpt_file_cer[PATH_MAX]="";
 
 char socket_dir[PATH_MAX];			/* where to talk to our citadel server */
-static const char editor_absolut_dir[PATH_MAX]=EDITORDIR;	/* nailed to what configure gives us. */
-static char static_dir[PATH_MAX];		/* calculated on startup */
-static char static_local_dir[PATH_MAX];		/* calculated on startup */
-static char static_icon_dir[PATH_MAX];          /* where should we find our mime icons? */
+const char editor_absolut_dir[PATH_MAX]=EDITORDIR;	/* nailed to what configure gives us. */
+char static_dir[PATH_MAX];		/* calculated on startup */
+char static_local_dir[PATH_MAX];		/* calculated on startup */
+char static_icon_dir[PATH_MAX];          /* where should we find our mime icons? */
 char  *static_dirs[]={				/* needs same sort order as the web mapping */
 	(char*)static_dir,			/* our templates on disk */
 	(char*)static_local_dir,		/* user provided templates disk */
@@ -74,307 +77,6 @@ char wizard_filename[PATH_MAX];	/* where's the setup wizard? */
 int running_as_daemon = 0;	/* should we deamonize on startup? */
 
 
-/* 
- * This is a generic function to set up a master socket for listening on
- * a TCP port.  The server shuts down if the bind fails.
- *
- * ip_addr 	IP address to bind
- * port_number	port number to bind
- * queue_len	number of incoming connections to allow in the queue
- */
-int ig_tcp_server(char *ip_addr, int port_number, int queue_len)
-{
-	struct sockaddr_in sin;
-	int s, i;
-
-	memset(&sin, 0, sizeof(sin));
-	sin.sin_family = AF_INET;
-	if (ip_addr == NULL) {
-		sin.sin_addr.s_addr = INADDR_ANY;
-	} else {
-		sin.sin_addr.s_addr = inet_addr(ip_addr);
-	}
-
-	if (sin.sin_addr.s_addr == INADDR_NONE) {
-		sin.sin_addr.s_addr = INADDR_ANY;
-	}
-
-	if (port_number == 0) {
-		lprintf(1, "Cannot start: no port number specified.\n");
-		exit(WC_EXIT_BIND);
-	}
-	sin.sin_port = htons((u_short) port_number);
-
-	s = socket(PF_INET, SOCK_STREAM, (getprotobyname("tcp")->p_proto));
-	if (s < 0) {
-		lprintf(1, "Can't create a socket: %s\n", strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-	/* Set some socket options that make sense. */
-	i = 1;
-	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
-
-	#ifndef __APPLE__
-	fcntl(s, F_SETFL, O_NONBLOCK); /* maide: this statement is incorrect
-					  there should be a preceding F_GETFL
-					  and a bitwise OR with the previous
-					  fd flags */
-	#endif
-	
-	if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-		lprintf(1, "Can't bind: %s\n", strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-	if (listen(s, queue_len) < 0) {
-		lprintf(1, "Can't listen: %s\n", strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-	return (s);
-}
-
-
-
-/*
- * Create a Unix domain socket and listen on it
- * sockpath - file name of the unix domain socket
- * queue_len - Number of incoming connections to allow in the queue
- */
-int ig_uds_server(char *sockpath, int queue_len)
-{
-	struct sockaddr_un addr;
-	int s;
-	int i;
-	int actual_queue_len;
-
-	actual_queue_len = queue_len;
-	if (actual_queue_len < 5) actual_queue_len = 5;
-
-	i = unlink(sockpath);
-	if ((i != 0) && (errno != ENOENT)) {
-		lprintf(1, "webcit: can't unlink %s: %s\n",
-			sockpath, strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-
-	memset(&addr, 0, sizeof(addr));
-	addr.sun_family = AF_UNIX;
-	safestrncpy(addr.sun_path, sockpath, sizeof addr.sun_path);
-
-	s = socket(AF_UNIX, SOCK_STREAM, 0);
-	if (s < 0) {
-		lprintf(1, "webcit: Can't create a socket: %s\n",
-			strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-
-	if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-		lprintf(1, "webcit: Can't bind: %s\n",
-			strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-
-	if (listen(s, actual_queue_len) < 0) {
-		lprintf(1, "webcit: Can't listen: %s\n",
-			strerror(errno));
-		exit(WC_EXIT_BIND);
-	}
-
-	chmod(sockpath, 0777);
-	return(s);
-}
-
-
-
-
-/*
- * Read data from the client socket.
- *
- * sock		socket fd to read from
- * buf		buffer to read into 
- * bytes	number of bytes to read
- * timeout	Number of seconds to wait before timing out
- *
- * Possible return values:
- *      1       Requested number of bytes has been read.
- *      0       Request timed out.
- *	-1   	Connection is broken, or other error.
- */
-int client_read_to(int *sock, StrBuf *Target, StrBuf *Buf, int bytes, int timeout)
-{
-	const char *Error;
-	int retval = 0;
-
-#ifdef HAVE_OPENSSL
-	if (is_https) {
-		while ((StrLength(Buf) + StrLength(Target) < bytes) &&
-		       (retval >= 0))
-			retval = client_read_sslbuffer(Buf, timeout);
-		if (retval >= 0) {
-			StrBufAppendBuf(Target, Buf, 0); /* todo: Buf > bytes? */
-#ifdef HTTP_TRACING
-			write(2, "\033[32m", 5);
-			write(2, buf, bytes);
-			write(2, "\033[30m", 5);
-#endif
-			return 1;
-		}
-		else {
-			lprintf(2, "client_read_ssl() failed\n");
-			return -1;
-		}
-	}
-#endif
-
-	if (StrLength(Buf) > 0) {/*/// todo: what if Buf > bytes?*/
-		StrBufAppendBuf(Target, Buf, 0);
-	}
-	retval = StrBufReadBLOB(Target, 
-			   sock, 
-			   (StrLength(Target) > 0), 
-			   bytes - StrLength(Target), 
-				&Error);
-	if (retval < 0) {
-		lprintf(2, "client_read() failed: %s\n",
-			Error);
-		return retval;
-	}
-
-#ifdef HTTP_TRACING
-	write(2, "\033[32m", 5);
-	write(2, buf, bytes);
-	write(2, "\033[30m", 5);
-#endif
-	return 1;
-}
-
-
-/*
- * Begin buffering HTTP output so we can transmit it all in one write operation later.
- */
-void begin_burst(void)
-{
-	if (WC->WBuf == NULL) {
-		WC->WBuf = NewStrBufPlain(NULL, 32768);
-	}
-}
-
-
-/*
- * Finish buffering HTTP output.  [Compress using zlib and] output with a Content-Length: header.
- */
-long end_burst(void)
-{
-	wcsession *WCC = WC;
-        const char *ptr, *eptr;
-        long count;
-	ssize_t res;
-        fd_set wset;
-        int fdflags;
-
-	//#ifdef HAVE_ZLIB
-	/* Perform gzip compression, if enabled and supported by client */
-	if (!DisableGzip && (WCC->gzip_ok) && CompressBuffer(WCC->WBuf))
-	{
-		hprintf("Content-encoding: gzip\r\n");
-	}
-	//#endif	/* HAVE_ZLIB */
-
-	hprintf("Content-length: %d\r\n\r\n", StrLength(WCC->WBuf));
-
-	ptr = ChrPtr(WCC->HBuf);
-	count = StrLength(WCC->HBuf);
-	eptr = ptr + count;
-
-#ifdef HAVE_OPENSSL
-	if (is_https) {
-		client_write_ssl(WCC->HBuf);
-		client_write_ssl(WCC->WBuf);
-		return (count);
-	}
-#endif
-
-	
-#ifdef HTTP_TRACING
-	
-	write(2, "\033[34m", 5);
-	write(2, ptr, StrLength(WCC->WBuf));
-	write(2, "\033[30m", 5);
-#endif
-	fdflags = fcntl(WC->http_sock, F_GETFL);
-
-	while (ptr < eptr) {
-                if ((fdflags & O_NONBLOCK) == O_NONBLOCK) {
-                        FD_ZERO(&wset);
-                        FD_SET(WCC->http_sock, &wset);
-                        if (select(WCC->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
-                                lprintf(2, "client_write: Socket select failed (%s)\n", strerror(errno));
-                                return -1;
-                        }
-                }
-
-                if ((res = write(WCC->http_sock, 
-				 ptr,
-				 count)) == -1) {
-                        lprintf(2, "client_write: Socket write failed (%s)\n", strerror(errno));
-		        wc_backtrace();
-                        return res;
-                }
-                count -= res;
-		ptr += res;
-        }
-
-	ptr = ChrPtr(WCC->WBuf);
-	count = StrLength(WCC->WBuf);
-	eptr = ptr + count;
-
-#ifdef HTTP_TRACING
-	
-	write(2, "\033[34m", 5);
-	write(2, ptr, StrLength(WCC->WBuf));
-	write(2, "\033[30m", 5);
-#endif
-
-        while (ptr < eptr) {
-                if ((fdflags & O_NONBLOCK) == O_NONBLOCK) {
-                        FD_ZERO(&wset);
-                        FD_SET(WCC->http_sock, &wset);
-                        if (select(WCC->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
-                                lprintf(2, "client_write: Socket select failed (%s)\n", strerror(errno));
-                                return -1;
-                        }
-                }
-
-                if ((res = write(WCC->http_sock, 
-				 ptr,
-				 count)) == -1) {
-                        lprintf(2, "client_write: Socket write failed (%s)\n", strerror(errno));
-			wc_backtrace();
-                        return res;
-                }
-                count -= res;
-		ptr += res;
-        }
-
-	return StrLength(WCC->WBuf);
-}
-
-
-
-/*
- * Read data from the client socket with default timeout.
- * (This is implemented in terms of client_read_to() and could be
- * justifiably moved out of sysdep.c)
- *
- * sock		the socket fd to read from
- * buf		the buffer to write to
- * bytes	Number of bytes to read
- */
-int client_read(int *sock, StrBuf *Target, StrBuf *buf, int bytes)
-{
-	return (client_read_to(sock, Target, buf, bytes, SLEEPING));
-}
-
-
 
 /*
  * Shut us down the regular way.
@@ -389,68 +91,6 @@ void graceful_shutdown_watcher(int signum) {
 }
 
 
-int ClientGetLine(int *sock, StrBuf *Target, StrBuf *CLineBuf)
-{
-	const char *Error, *pch, *pchs;
-	int rlen, len, retval = 0;
-
-#ifdef HAVE_OPENSSL
-	if (is_https) {
-		int ntries = 0;
-		if (StrLength(CLineBuf) > 0) {
-			pchs = ChrPtr(CLineBuf);
-			pch = strchr(pchs, '\n');
-			if (pch != NULL) {
-				rlen = 0;
-				len = pch - pchs;
-				if (len > 0 && (*(pch - 1) == '\r') )
-					rlen ++;
-				StrBufSub(Target, CLineBuf, 0, len - rlen);
-				StrBufCutLeft(CLineBuf, len + 1);
-				return len - rlen;
-			}
-		}
-
-		while (retval == 0) { 
-				pch = NULL;
-				pchs = ChrPtr(CLineBuf);
-				if (*pchs != '\0')
-					pch = strchr(pchs, '\n');
-				if (pch == NULL) {
-					retval = client_read_sslbuffer(CLineBuf, SLEEPING);
-					pchs = ChrPtr(CLineBuf);
-					pch = strchr(pchs, '\n');
-				}
-				if (retval == 0) {
-					sleeeeeeeeeep(1);
-					ntries ++;
-				}
-				if (ntries > 10)
-					return 0;
-		}
-		if ((retval > 0) && (pch != NULL)) {
-			rlen = 0;
-			len = pch - pchs;
-			if (len > 0 && (*(pch - 1) == '\r') )
-				rlen ++;
-			StrBufSub(Target, CLineBuf, 0, len - rlen);
-			StrBufCutLeft(CLineBuf, len + 1);
-			return len - rlen;
-
-		}
-		else 
-			return -1;
-	}
-	else 
-#endif
-		return StrBufTCP_read_buffered_line(Target, 
-						    CLineBuf,
-						    sock,
-						    5,
-						    1,
-						    &Error);
-}
-
 
 
 /*
@@ -459,11 +99,10 @@ int ClientGetLine(int *sock, StrBuf *Target, StrBuf *CLineBuf)
  */
 pid_t current_child;
 void graceful_shutdown(int signum) {
-	char wd[SIZ];
 	FILE *FD;
 	int fd;
-	getcwd(wd, SIZ);
-	lprintf (1, "bye going down gracefull.[%d][%s]\n", signum, wd);
+
+	lprintf (1, "WebCit is being shut down on signal %d.\n", signum);
 	fd = msock;
 	msock = -1;
 	time_to_die = 1;
@@ -483,6 +122,8 @@ void start_daemon(char *pid_file)
 	pid_t child = 0;
 	FILE *fp;
 	int do_restart = 0;
+	int rv;
+	FILE *rvfp = NULL;
 
 	current_child = 0;
 
@@ -490,7 +131,7 @@ void start_daemon(char *pid_file)
 	 * We don't just call close() because we don't want these fd's
 	 * to be reused for other files.
 	 */
-	chdir("/");
+	rv = chdir("/");
 
 	signal(SIGHUP, SIG_IGN);
 	signal(SIGINT, SIG_IGN);
@@ -503,9 +144,9 @@ void start_daemon(char *pid_file)
 
 	setsid();
 	umask(0);
-	freopen("/dev/null", "r", stdin);
-	freopen("/dev/null", "w", stdout);
-	freopen("/dev/null", "w", stderr);
+	rvfp = freopen("/dev/null", "r", stdin);
+	rvfp = freopen("/dev/null", "w", stdout);
+	rvfp = freopen("/dev/null", "w", stderr);
 	signal(SIGTERM, graceful_shutdown_watcher);
 	signal(SIGHUP, graceful_shutdown_watcher);
 
@@ -579,16 +220,15 @@ void spawn_another_worker_thread()
 	pthread_attr_t attr;	/* Thread attributes */
 	int ret;
 
-	lprintf(3, "Creating a new thread\n");
+	lprintf(3, "Creating a new thread.  Pool size is now %d\n", ++num_threads);
 
 	/* set attributes for the new thread */
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 
 	/*
-	 * Our per-thread stacks need to be bigger than the default size, otherwise
-	 * the MIME parser crashes on FreeBSD, and the IMAP service crashes on
-	 * 64-bit Linux.
+	 * Our per-thread stacks need to be bigger than the default size,
+	 * otherwise the MIME parser crashes on FreeBSD.
 	 */
 	if ((ret = pthread_attr_setstacksize(&attr, 1024 * 1024))) {
 		lprintf(1, "pthread_attr_setstacksize: %s\n",
@@ -618,19 +258,69 @@ extern int DumpTemplateI18NStrings;
 extern StrBuf *I18nDump;
 void InitTemplateCache(void);
 extern int LoadTemplates;
-extern void LoadZoneFiles(void);
-StrBuf *csslocal = NULL;
+
+extern HashList *HandlerHash;
+
+
+
+void
+webcit_calc_dirs_n_files(int relh, const char *basedir, int home, char *webcitdir, char *relhome)
+{
+	char dirbuffer[PATH_MAX]="";
+	/* calculate all our path on a central place */
+    /* where to keep our config */
+	
+#define COMPUTE_DIRECTORY(SUBDIR) memcpy(dirbuffer,SUBDIR, sizeof dirbuffer);\
+	snprintf(SUBDIR,sizeof SUBDIR,  "%s%s%s%s%s%s%s", \
+			 (home&!relh)?webcitdir:basedir, \
+             ((basedir!=webcitdir)&(home&!relh))?basedir:"/", \
+             ((basedir!=webcitdir)&(home&!relh))?"/":"", \
+			 relhome, \
+             (relhome[0]!='\0')?"/":"",\
+			 dirbuffer,\
+			 (dirbuffer[0]!='\0')?"/":"");
+	basedir=RUNDIR;
+	COMPUTE_DIRECTORY(socket_dir);
+	basedir=WWWDIR "/static";
+	COMPUTE_DIRECTORY(static_dir);
+	basedir=WWWDIR "/static/icons";
+	COMPUTE_DIRECTORY(static_icon_dir);
+	basedir=WWWDIR "/static.local";
+	COMPUTE_DIRECTORY(static_local_dir);
+	StripSlashes(static_dir, 1);
+	StripSlashes(static_icon_dir, 1);
+	StripSlashes(static_local_dir, 1);
+
+	snprintf(file_crpt_file_key,
+		 sizeof file_crpt_file_key, 
+		 "%s/citadel.key",
+		 ctdl_key_dir);
+	snprintf(file_crpt_file_csr,
+		 sizeof file_crpt_file_csr, 
+		 "%s/citadel.csr",
+		 ctdl_key_dir);
+	snprintf(file_crpt_file_cer,
+		 sizeof file_crpt_file_cer, 
+		 "%s/citadel.cer",
+		 ctdl_key_dir);
+
+	/* we should go somewhere we can leave our coredump, if enabled... */
+	lprintf(9, "Changing directory to %s\n", socket_dir);
+	if (chdir(webcitdir) != 0) {
+		perror("chdir");
+	}
+}
 /*
  * Here's where it all begins.
  */
 int main(int argc, char **argv)
 {
+	size_t basesize = 2;            /* how big should strbufs be on creation? */
 	pthread_t SessThread;		/* Thread descriptor */
 	pthread_attr_t attr;		/* Thread attributes */
 	int a, i;	        	/* General-purpose variables */
 	char tracefile[PATH_MAX];
 	char ip_addr[256]="0.0.0.0";
-	char dirbuffer[PATH_MAX]="";
 	int relh=0;
 	int home=0;
 	int home_specified=0;
@@ -638,33 +328,18 @@ int main(int argc, char **argv)
 	char webcitdir[PATH_MAX] = DATADIR;
 	char *pidfile = NULL;
 	char *hdir;
-	const char *basedir;
-#ifdef ENABLE_NLS
-	char *locale = NULL;
-	char *mo = NULL;
-#endif /* ENABLE_NLS */
+	const char *basedir = NULL;
 	char uds_listen_path[PATH_MAX];	/* listen on a unix domain socket? */
-	const char *I18nDumpFile;
+	const char *I18nDumpFile = NULL;
+	FILE *rvfp = NULL;
+	int rv = 0;
 
 	WildFireInitBacktrace(argv[0], 2);
 
-	HandlerHash = NewHash(1, NULL);
-	PreferenceHooks = NewHash(1, NULL);
-	WirelessTemplateCache = NewHash(1, NULL);
-	WirelessLocalTemplateCache = NewHash(1, NULL);
-	LocalTemplateCache = NewHash(1, NULL);
-	TemplateCache = NewHash(1, NULL);
-	GlobalNS = NewHash(1, NULL);
-	Iterators = NewHash(1, NULL);
-	Conditionals = NewHash(1, NULL);
-	MsgHeaderHandler = NewHash(1, NULL);
-	MimeRenderHandler = NewHash(1, NULL);
-	SortHash = NewHash(1, NULL);
-
-	LoadZoneFiles();
+	start_modules ();
 
 #ifdef DBG_PRINNT_HOOKS_AT_START
-	dbg_PrintHash(HandlerHash, nix, NULL);
+/*	dbg_PrintHash(HandlerHash, nix, NULL);*/
 #endif
 
 	/* Ensure that we are linked to the correct version of libcitadel */
@@ -680,19 +355,20 @@ int main(int argc, char **argv)
 
 	/* Parse command line */
 #ifdef HAVE_OPENSSL
-	while ((a = getopt(argc, argv, "h:i:p:t:T:x:dD:G:cfsZ")) != EOF)
+	while ((a = getopt(argc, argv, "h:i:p:t:T:B:x:dD:G:cfsS:Z")) != EOF)
 #else
-	while ((a = getopt(argc, argv, "h:i:p:t:T:x:dD:G:cfZ")) != EOF)
+	while ((a = getopt(argc, argv, "h:i:p:t:T:B:x:dD:G:cfZ")) != EOF)
 #endif
 		switch (a) {
 		case 'h':
 			hdir = strdup(optarg);
 			relh=hdir[0]!='/';
-			if (!relh) safestrncpy(webcitdir, hdir,
-								   sizeof webcitdir);
-			else
-				safestrncpy(relhome, relhome,
-							sizeof relhome);
+			if (!relh) {
+				safestrncpy(webcitdir, hdir, sizeof webcitdir);
+			}
+			else {
+				safestrncpy(relhome, relhome, sizeof relhome);
+			}
 			/* free(hdir); TODO: SHOULD WE DO THIS? */
 			home_specified = 1;
 			home=1;
@@ -704,6 +380,11 @@ int main(int argc, char **argv)
 			pidfile = strdup(optarg);
 			running_as_daemon = 1;
 			break;
+		case 'B': /* Basesize */
+			basesize = atoi(optarg);
+			if (basesize > 2)
+				StartLibCitadel(basesize);
+			break;
 		case 'i':
 			safestrncpy(ip_addr, optarg, sizeof ip_addr);
 			break;
@@ -715,9 +396,9 @@ int main(int argc, char **argv)
 			break;
 		case 't':
 			safestrncpy(tracefile, optarg, sizeof tracefile);
-			freopen(tracefile, "w", stdout);
-			freopen(tracefile, "w", stderr);
-			freopen(tracefile, "r", stdin);
+			rvfp = freopen(tracefile, "w", stdout);
+			rvfp = freopen(tracefile, "w", stderr);
+			rvfp = freopen(tracefile, "r", stdin);
 			break;
 		case 'T':
 			LoadTemplates = atoi(optarg);
@@ -748,9 +429,15 @@ int main(int argc, char **argv)
 				}
 			}
 			break;
+#ifdef HAVE_OPENSSL
 		case 's':
 			is_https = 1;
 			break;
+		case 'S':
+			is_https = 1;
+			ssl_cipher_list = strdup(optarg);
+			break;
+#endif
 		case 'G':
 			DumpTemplateI18NStrings = 1;
 			I18nDump = NewStrBufPlain(HKEY("int templatestrings(void)\n{\n"));
@@ -763,7 +450,7 @@ int main(int argc, char **argv)
 				"[-T Templatedebuglevel] "
 				"[-d] [-Z] [-G i18ndumpfile] "
 #ifdef HAVE_OPENSSL
-				"[-s] "
+				"[-s] [-S cipher_suites]"
 #endif
 				"[remotehost [remoteport]]\n");
 			return 1;
@@ -783,6 +470,9 @@ int main(int argc, char **argv)
 		signal(SIGHUP, graceful_shutdown);
 	}
 
+	webcit_calc_dirs_n_files(relh, basedir, home, webcitdir, relhome);
+	LoadIconDir(static_icon_dir);
+
 	/* Tell 'em who's in da house */
 	lprintf(1, PACKAGE_STRING "\n");
 	lprintf(1, "Copyright (C) 1996-2009 by the Citadel development team.\n"
@@ -792,60 +482,6 @@ int main(int argc, char **argv)
 
 
 	/* initialize the International Bright Young Thing */
-#ifdef ENABLE_NLS
-	initialize_locales();
-
-
-	locale = setlocale(LC_ALL, "");
-
-	mo = malloc(strlen(webcitdir) + 20);
-	lprintf(9, "Message catalog directory: %s\n", bindtextdomain("webcit", LOCALEDIR"/locale"));
-	free(mo);
-	lprintf(9, "Text domain: %s\n", textdomain("webcit"));
-	lprintf(9, "Text domain Charset: %s\n", bind_textdomain_codeset("webcit","UTF8"));
-#endif
-
-
-	/* calculate all our path on a central place */
-    /* where to keep our config */
-	
-#define COMPUTE_DIRECTORY(SUBDIR) memcpy(dirbuffer,SUBDIR, sizeof dirbuffer);\
-	snprintf(SUBDIR,sizeof SUBDIR,  "%s%s%s%s%s%s%s", \
-			 (home&!relh)?webcitdir:basedir, \
-             ((basedir!=webcitdir)&(home&!relh))?basedir:"/", \
-             ((basedir!=webcitdir)&(home&!relh))?"/":"", \
-			 relhome, \
-             (relhome[0]!='\0')?"/":"",\
-			 dirbuffer,\
-			 (dirbuffer[0]!='\0')?"/":"");
-	basedir=RUNDIR;
-	COMPUTE_DIRECTORY(socket_dir);
-	basedir=WWWDIR "/static";
-	COMPUTE_DIRECTORY(static_dir);
-	basedir=WWWDIR "/static/icons";
-	COMPUTE_DIRECTORY(static_icon_dir);
-	basedir=WWWDIR "/static.local";
-	COMPUTE_DIRECTORY(static_local_dir);
-
-	snprintf(file_crpt_file_key,
-		 sizeof file_crpt_file_key, 
-		 "%s/citadel.key",
-		 ctdl_key_dir);
-	snprintf(file_crpt_file_csr,
-		 sizeof file_crpt_file_csr, 
-		 "%s/citadel.csr",
-		 ctdl_key_dir);
-	snprintf(file_crpt_file_cer,
-		 sizeof file_crpt_file_cer, 
-		 "%s/citadel.cer",
-		 ctdl_key_dir);
-
-	/* we should go somewhere we can leave our coredump, if enabled... */
-	lprintf(9, "Changing directory to %s\n", socket_dir);
-	if (chdir(webcitdir) != 0) {
-		perror("chdir");
-	}
-	LoadIconDir(static_icon_dir);
 
 	initialise_modules();
 	initialize_viewdefs();
@@ -868,14 +504,11 @@ int main(int argc, char **argv)
 			lprintf(1, "********************************************************************************\n");
 			return -1;
 		}
-		fwrite(ChrPtr(I18nDump), 1, StrLength(I18nDump), fd);
+		rv = fwrite(ChrPtr(I18nDump), 1, StrLength(I18nDump), fd);
 		fclose(fd);
 		return 0;
 	}
 
-	if (!access("static.local/webcit.css", R_OK)) {
-		csslocal = NewStrBufPlain(HKEY("<link href=\"static.local/webcit.css\" rel=\"stylesheet\" type=\"text/css\">"));
-	}
 
 	/* Tell libical to return an error instead of aborting if it sees badly formed iCalendar data. */
 	icalerror_errors_are_fatal = 0;
@@ -919,6 +552,11 @@ int main(int argc, char **argv)
 		lprintf(2, "Attempting to bind to port %d...\n", http_port);
 		msock = ig_tcp_server(ip_addr, http_port, LISTEN_QUEUE_LENGTH);
 	}
+	if (msock < 0)
+	{
+		ShutDownWebcit();
+		return -msock;
+	}
 
 	lprintf(2, "Listening on socket %d\n", msock);
 	signal(SIGPIPE, SIG_IGN);
@@ -951,34 +589,17 @@ int main(int argc, char **argv)
 	/* now the original thread becomes another worker */
 	worker_entry();
 	ShutDownLibCitadel ();
-	DeleteHash(&HandlerHash);
-	DeleteHash(&PreferenceHooks);
 	return 0;
 }
 
 
 void ShutDownWebcit(void)
 {
-	DeleteHash(&ZoneHash);
 	free_zone_directory ();
 	icaltimezone_release_zone_tab ();
 	icalmemory_free_ring ();
 	ShutDownLibCitadel ();
-	DeleteHash(&HandlerHash);
-	DeleteHash(&PreferenceHooks);
-	DeleteHash(&GlobalNS);
-	DeleteHash(&WirelessTemplateCache);
-	DeleteHash(&WirelessLocalTemplateCache);
-	DeleteHash(&TemplateCache);
-	DeleteHash(&LocalTemplateCache);
-	DeleteHash(&Iterators);
-	DeleteHash(&MimeRenderHandler);
-	DeleteHash(&Conditionals);
-	DeleteHash(&MsgHeaderHandler);
-	DeleteHash(&SortHash);
-#ifdef ENABLE_NLS
-	ShutdownLocale();
-#endif
+	shutdown_modules ();
 #ifdef HAVE_OPENSSL
 	if (is_https) {
 		shutdown_ssl();
@@ -997,7 +618,11 @@ void worker_entry(void)
 	int ret;
 	struct timeval tv;
 	fd_set readset, tempset;
+	ParsedHttpHdrs Hdr;
 
+	memset(&Hdr, 0, sizeof(ParsedHttpHdrs));
+	Hdr.HR.eReqType = eGET;
+	http_new_modules(&Hdr);	
 	tv.tv_sec = 0;
 	tv.tv_usec = 10000;
 	FD_ZERO(&readset);
@@ -1046,6 +671,7 @@ void worker_entry(void)
 			end_critical_section(S_SHUTDOWN);
 			if (shutdown == 1)
 			{/* we're the one to cleanup the mess. */
+				http_destroy_modules(&Hdr);
 				lprintf(2, "I'm master shutdown: tagging sessions to be killed.\n");
 				shutdown_sessions();
 				lprintf(2, "master shutdown: waiting for others\n");
@@ -1081,12 +707,25 @@ void worker_entry(void)
 					close(ssock);
 				}
 			}
+			else 
 #endif
+			{
+				int fdflags; 
+				fdflags = fcntl(ssock, F_GETFL);
+				if (fdflags < 0)
+					lprintf(1, "unable to get server socket flags! %s \n",
+						strerror(errno));
+				fdflags = fdflags | O_NONBLOCK;
+				if (fcntl(ssock, F_SETFL, fdflags) < 0)
+					lprintf(1, "unable to set server socket nonblocking flags! %s \n",
+						strerror(errno));
+			}
 
 			if (fail_this_transaction == 0) {
+				Hdr.http_sock = ssock;
 
 				/* Perform an HTTP transaction... */
-				context_loop(&ssock);
+				context_loop(&Hdr);
 
 				/* Shut down SSL/TLS if required... */
 #ifdef HAVE_OPENSSL
@@ -1096,14 +735,17 @@ void worker_entry(void)
 #endif
 
 				/* ...and close the socket. */
-				if (ssock > 0)
+				if (Hdr.http_sock > 0)
 					lingering_close(ssock);
+				http_detach_modules(&Hdr);
+
 			}
 
 		}
 
 	} while (!time_to_die);
 
+	http_destroy_modules(&Hdr);
 	lprintf (1, "bye\n");
 	pthread_exit(NULL);
 }
@@ -1153,3 +795,6 @@ void wc_backtrace(void)
 #endif
 }
 
+
+
+
